create or replace
PACKAGE BODY PLOG_OUT_SESSION AS

--*******************************************************************************
--   NAME:   PLOG_OUT_SESSION (body)
--
--   writes the log information into the view V$SESSION trough the public procedure log()
--
--   Ver    Date        Autor             Comment
--   -----  ----------  ----------------  ---------------------------------------
--   1.0    14.04.2008  Bertrand Caradec  First version
--*******************************************************************************

  PROCEDURE log
(
    pCTX        IN       PLOGPARAM.LOG_CTX                ,  
    pID         IN       TLOG.id%TYPE                      ,
    pLDate      IN       TLOG.ldate%TYPE                   ,
    pLHSECS     IN       TLOG.lhsecs%TYPE                  ,
    pLLEVEL     IN       TLOG.llevel%TYPE                  ,
    pLSECTION   IN       TLOG.lsection%TYPE                ,
    pLUSER      IN       TLOG.luser%TYPE                   ,
    pLTEXT      IN       TLOG.LTEXT%TYPE
) 
--*******************************************************************************
--   NAME:   log
--
--   PARAMETERS:
--
--      pCTX               log context
--      pID                ID of the log message, generated by the sequence
--      pLDate             Date of the log message (SYSDATE)
--      pLHSECS            Number of seconds since the beginning of the epoch
--      pLLEVEL            Log level as numeric value
--      pLSection          formated call stack
--      pLUSER             database user (SYSUSER)
--      pLTEXT             log text
--
--   Public. Write the log information into the view V$SESSION.
--
--   Ver    Date        Autor             Comment
--   -----  ----------  ---------------   ----------------------------------------
--   1.0    14.04.2008  Bertrand Caradec  Initial version
--*******************************************************************************

AS
BEGIN
    IF pCTX.USE_SESSION = TRUE THEN        
       dbms_application_info.set_module(pLSECTION || ' User:' || PLUSER, NULL);        
       dbms_application_info.set_action(PLOGPARAM.getLevelInText(pLLEVEL) || ' ' ||  to_char(pLDATE, 'DD.MM.YYYY HH24:MI:SS') ); 
       dbms_application_info.set_client_info(pLTEXT);       
       
   END IF;
END log;

-- end of the package
END;
/
