--------------------------------------------------------
--  File created - Sekmadienis-balandis-12-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package HTTP
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "HTTP" as 
  
  /*
    Wrapper package around UTL_HTTP to make HTTP 1.1 GET and POST requests.
    
    Depends on:
    UTL_HTTP
  */ 
    
  type t_headers is table of varchar2(500) index by varchar2(200);

  /*
    HTTP 1.1 GET request
  */
  function do_get_request(
    p_url in varchar2,
    p_request_headers in t_headers) return varchar2;
  
  /*
    HTTP 1.1 POST request for up to 32K requests and up to 32K responses.
    At least "Content-Length" HTTP header MUST BE supplied to this function.
  */
  function do_post_request(
    p_url in varchar2,
    p_request_headers in t_headers,
    p_request_body in varchar2) return varchar2;

  /*
    HTTP 1.1 POST request for >32K requests and >32K responses.
    At least "Content-Length" HTTP header MUST BE supplied to this function.
    Function adds "Transfer-Encoding" header if request body is greater than
    32K.
  */
  function do_post_request(
    p_url in varchar2,
    p_request_headers in t_headers,
    p_request_body in clob) return clob;
    
  /*
    HTTP 1.1 PUT request for up to 32K requests and up to 32K responses.
    At least "Content-Length" HTTP header MUST BE supplied to this function.
  */
  function do_put_request(
    p_url in varchar2,
    p_request_headers in t_headers,
    p_request_body in varchar2) return varchar2;

end http;

/
--------------------------------------------------------
--  DDL for Package JSON_AC
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "JSON_AC" as
  --json type methods

  procedure object_remove(p_self in out nocopy json, pair_name varchar2);
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value json_value, position pls_integer default null);
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value varchar2, position pls_integer default null);
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value number, position pls_integer default null);
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value boolean, position pls_integer default null);
  procedure object_check_duplicate(p_self in out nocopy json, v_set boolean);
  procedure object_remove_duplicates(p_self in out nocopy json);

  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value json, position pls_integer default null);
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value json_list, position pls_integer default null);

  function object_count(p_self in json) return number;
  function object_get(p_self in json, pair_name varchar2) return json_value;
  function object_get(p_self in json, position pls_integer) return json_value;
  function object_index_of(p_self in json, pair_name varchar2) return number;
  function object_exist(p_self in json, pair_name varchar2) return boolean;

  function object_to_char(p_self in json, spaces boolean default true, chars_per_line number default 0) return varchar2;
  procedure object_to_clob(p_self in json, buf in out nocopy clob, spaces boolean default false, chars_per_line number default 0, erase_clob boolean default true);
  procedure object_print(p_self in json, spaces boolean default true, chars_per_line number default 8192, jsonp varchar2 default null);
  procedure object_htp(p_self in json, spaces boolean default false, chars_per_line number default 0, jsonp varchar2 default null);

  function object_to_json_value(p_self in json) return json_value;
  function object_path(p_self in json, json_path varchar2, base number default 1) return json_value;

  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem json_value, base number default 1);
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem varchar2  , base number default 1);
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem number    , base number default 1);
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem boolean   , base number default 1);
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem json_list , base number default 1);
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem json      , base number default 1);

  procedure object_path_remove(p_self in out nocopy json, json_path varchar2, base number default 1);

  function object_get_values(p_self in json) return json_list;
  function object_get_keys(p_self in json) return json_list;

  --json_list type methods
  procedure array_append(p_self in out nocopy json_list, elem json_value, position pls_integer default null);
  procedure array_append(p_self in out nocopy json_list, elem varchar2, position pls_integer default null);
  procedure array_append(p_self in out nocopy json_list, elem number, position pls_integer default null);
  procedure array_append(p_self in out nocopy json_list, elem boolean, position pls_integer default null);
  procedure array_append(p_self in out nocopy json_list, elem json_list, position pls_integer default null);

  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem json_value);
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem varchar2);
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem number);
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem boolean);
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem json_list);

  function array_count(p_self in json_list) return number;
  procedure array_remove(p_self in out nocopy json_list, position pls_integer);
  procedure array_remove_first(p_self in out nocopy json_list);
  procedure array_remove_last(p_self in out nocopy json_list);
  function array_get(p_self in json_list, position pls_integer) return json_value;
  function array_head(p_self in json_list) return json_value;
  function array_last(p_self in json_list) return json_value;
  function array_tail(p_self in json_list) return json_list;

  function array_to_char(p_self in json_list, spaces boolean default true, chars_per_line number default 0) return varchar2;
  procedure array_to_clob(p_self in json_list, buf in out nocopy clob, spaces boolean default false, chars_per_line number default 0, erase_clob boolean default true);
  procedure array_print(p_self in json_list, spaces boolean default true, chars_per_line number default 8192, jsonp varchar2 default null);
  procedure array_htp(p_self in json_list, spaces boolean default false, chars_per_line number default 0, jsonp varchar2 default null);

  function array_path(p_self in json_list, json_path varchar2, base number default 1) return json_value;
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem json_value, base number default 1);
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem varchar2  , base number default 1);
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem number    , base number default 1);
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem boolean   , base number default 1);
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem json_list , base number default 1);

  procedure array_path_remove(p_self in out nocopy json_list, json_path varchar2, base number default 1);

  function array_to_json_value(p_self in json_list) return json_value;

  --json_value


  function jv_get_type(p_self in json_value) return varchar2;
  function jv_get_string(p_self in json_value, max_byte_size number default null, max_char_size number default null) return varchar2;
  procedure jv_get_string(p_self in json_value, buf in out nocopy clob);
  function jv_get_number(p_self in json_value) return number;
  function jv_get_bool(p_self in json_value) return boolean;
  function jv_get_null(p_self in json_value) return varchar2;

  function jv_is_object(p_self in json_value) return boolean;
  function jv_is_array(p_self in json_value) return boolean;
  function jv_is_string(p_self in json_value) return boolean;
  function jv_is_number(p_self in json_value) return boolean;
  function jv_is_bool(p_self in json_value) return boolean;
  function jv_is_null(p_self in json_value) return boolean;

  function jv_to_char(p_self in json_value, spaces boolean default true, chars_per_line number default 0) return varchar2;
  procedure jv_to_clob(p_self in json_value, buf in out nocopy clob, spaces boolean default false, chars_per_line number default 0, erase_clob boolean default true);
  procedure jv_print(p_self in json_value, spaces boolean default true, chars_per_line number default 8192, jsonp varchar2 default null);
  procedure jv_htp(p_self in json_value, spaces boolean default false, chars_per_line number default 0, jsonp varchar2 default null);

  function jv_value_of(p_self in json_value, max_byte_size number default null, max_char_size number default null) return varchar2;


end json_ac;

/
--------------------------------------------------------
--  DDL for Package JSON_EXT
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "JSON_EXT" as
  /*
  Copyright (c) 2009 Jonas Krogsboell

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  */

  /* This package contains extra methods to lookup types and
     an easy way of adding date values in json - without changing the structure */
  function parsePath(json_path varchar2, base number default 1) return json_list;

  --JSON Path getters
  function get_json_value(obj json, v_path varchar2, base number default 1) return json_value;
  function get_string(obj json, path varchar2,       base number default 1) return varchar2;
  function get_number(obj json, path varchar2,       base number default 1) return number;
  function get_json(obj json, path varchar2,         base number default 1) return json;
  function get_json_list(obj json, path varchar2,    base number default 1) return json_list;
  function get_bool(obj json, path varchar2,         base number default 1) return boolean;

  --JSON Path putters
  procedure put(obj in out nocopy json, path varchar2, elem varchar2,   base number default 1);
  procedure put(obj in out nocopy json, path varchar2, elem number,     base number default 1);
  procedure put(obj in out nocopy json, path varchar2, elem json,       base number default 1);
  procedure put(obj in out nocopy json, path varchar2, elem json_list,  base number default 1);
  procedure put(obj in out nocopy json, path varchar2, elem boolean,    base number default 1);
  procedure put(obj in out nocopy json, path varchar2, elem json_value, base number default 1);

  procedure remove(obj in out nocopy json, path varchar2, base number default 1);

  --Pretty print with JSON Path - obsolete in 0.9.4 - obj.path(v_path).(to_char,print,htp)
  function pp(obj json, v_path varchar2) return varchar2;
  procedure pp(obj json, v_path varchar2); --using dbms_output.put_line
  procedure pp_htp(obj json, v_path varchar2); --using htp.print

  --extra function checks if number has no fraction
  function is_integer(v json_value) return boolean;

  format_string varchar2(30 char) := 'yyyy-mm-dd hh24:mi:ss';
  --extension enables json to store dates without comprimising the implementation
  function to_json_value(d date) return json_value;
  --notice that a date type in json is also a varchar2
  function is_date(v json_value) return boolean;
  --convertion is needed to extract dates
  --(json_ext.to_date will not work along with the normal to_date function - any fix will be appreciated)
  function to_date2(v json_value) return date;
  --JSON Path with date
  function get_date(obj json, path varchar2, base number default 1) return date;
  procedure put(obj in out nocopy json, path varchar2, elem date, base number default 1);

  --experimental support of binary data with base64
  function base64(binarydata blob) return json_list;
  function base64(l json_list) return blob;

  function encode(binarydata blob) return json_value;
  function decode(v json_value) return blob;

end json_ext;

/
--------------------------------------------------------
--  DDL for Package JSON_PARSER
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "JSON_PARSER" as
  /*
  Copyright (c) 2010 Jonas Krogsboell

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  */
  /* scanner tokens:
    '{', '}', ',', ':', '[', ']', STRING, NUMBER, TRUE, FALSE, NULL
  */
  type rToken IS RECORD (
    type_name VARCHAR2(7),
    line PLS_INTEGER,
    col PLS_INTEGER,
    data VARCHAR2(32767),
    data_overflow clob); -- max_string_size

  type lTokens is table of rToken index by pls_integer;
  type json_src is record (len number, offset number, src varchar2(32767), s_clob clob);

  json_strict boolean not null := false;

  function next_char(indx number, s in out nocopy json_src) return varchar2;
  function next_char2(indx number, s in out nocopy json_src, amount number default 1) return varchar2;

  function prepareClob(buf in clob) return json_parser.json_src;
  function prepareVarchar2(buf in varchar2) return json_parser.json_src;
  function lexer(jsrc in out nocopy json_src) return lTokens;
  procedure print_token(t rToken);

  function parser(str varchar2) return json;
  function parse_list(str varchar2) return json_list;
  function parse_any(str varchar2) return json_value;
  function parser(str clob) return json;
  function parse_list(str clob) return json_list;
  function parse_any(str clob) return json_value;
  procedure remove_duplicates(obj in out nocopy json);
  function get_version return varchar2;

end json_parser;

/
--------------------------------------------------------
--  DDL for Package JSON_PRINTER
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "JSON_PRINTER" as
  /*
  Copyright (c) 2010 Jonas Krogsboell

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  */
  indent_string varchar2(10 char) := '  '; --chr(9); for tab
  newline_char varchar2(2 char)   := chr(13)||chr(10); -- Windows style
  --newline_char varchar2(2) := chr(10); -- Mac style
  --newline_char varchar2(2) := chr(13); -- Linux style
  ascii_output boolean    not null := true;
  escape_solidus boolean  not null := false;

  function pretty_print(obj json, spaces boolean default true, line_length number default 0) return varchar2;
  function pretty_print_list(obj json_list, spaces boolean default true, line_length number default 0) return varchar2;
  function pretty_print_any(json_part json_value, spaces boolean default true, line_length number default 0) return varchar2;
  procedure pretty_print(obj json, spaces boolean default true, buf in out nocopy clob, line_length number default 0, erase_clob boolean default true);
  procedure pretty_print_list(obj json_list, spaces boolean default true, buf in out nocopy clob, line_length number default 0, erase_clob boolean default true);
  procedure pretty_print_any(json_part json_value, spaces boolean default true, buf in out nocopy clob, line_length number default 0, erase_clob boolean default true);

  procedure dbms_output_clob(my_clob clob, delim varchar2, jsonp varchar2 default null);
  procedure htp_output_clob(my_clob clob, jsonp varchar2 default null);
end json_printer;

/
--------------------------------------------------------
--  DDL for Package RDM_AUTH
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_AUTH" is

/*
  Paketas grupuoja rašto darbų mainavietės autentifikavimo ir autorizavimo logiką.
  Sistemoje įgyvendintas SAML kalba paremtas SSO sprendimas naudojant Shibboleth
  programinę įrangą. Su Shibboleth susijusi logika įgyvendinta pakete shibboleth.

  Prisijungimo scenarijus:
  1. Naudotojui bandant pasinaudoti sistemos funkcionalumu, jis pirmiausia 
  nukreipiamas į SSO prisijungimo langą. Tai atlieka Apache mod_shib modulis.
  2. Naudotojui sėkmingai autentifikavusis, SSO jį nukreipia į APEX aplikaciją.
  3. Kadangi APEX aplikacijoje naudotojas dar neautentifikuotas, APEX nukreipia
     į prisijungimo puslapį (mainavietės atveju tai yra 6 puslapis).
  4. APEX prisijungimo puslapyje atliekami tokie žingsniai:
  4.1. Gaunami Shibboleth atributai - kreipiamasi į shibboleth.get_attributes.
  4.2. Gaunamas autorizavimo kontekstas - kreipiamasi į rdm_auth.get_context.
       Kontekste yra visi aplikacijos autorizavimui reikalingi duomenys. 
       Kontekstas užpildomas iš Shibboleth atributų.
  4.3. Atliekamas naudotojo prijungimas prie sistemos - kreipiamasi į rdm_auth.login.
       Kadangi naudotojas jau yra autentifikuotas, login f-ja grąžina true, tai yra,
       visada prijungia.
*/

/*
  Rašto darbų mainavietės rolės
*/
C_STUDENT constant varchar2(20) := 'STUDENT';                     /* Studentas */
C_TUTOR constant varchar2(20) := 'TUTOR';                         /* Dėstytojas */
C_DEPARTMENT_ADM constant varchar2(20) := 'DEPARTMENT_ADM';       /* Padalinio administratorius */
C_INSTITUTION_ADM constant varchar2(20) := 'INSTITUTION_ADM';     /* Institucijos administratorius */

/*
  eLABa rolės, aktualios rašto darbų mainavietei
*/
C_DOC_SUP_EDIT_EPAS constant varchar2(35) := 'DOCUMENT__SUPERVISOR__EDIT_EPAS';
C_DOC_REV_EDIT_EPAS constant varchar2(35) := 'DOCUMENT__REVIEWER__EDIT_EPAS';
C_DOC_DEP_EDIT_EPAS constant varchar2(35) := 'DOCUMENT__DEPARTMENT__EDIT_EPAS';
C_DOC_INS_EDIT_EPAS constant varchar2(35) := 'DOCUMENT__INSTITUTION__EDIT_EPAS';

/*
  Rolių sąrašas
*/
type t_roles is table of varchar2(1) index by varchar2(20);

/*
  Padalinių sąrašas. eLABa SSO bendru atveju gali perduoti daugiau nei vieną
  padalinį: pad1;pad2;pad3;...;padN. Šiame masyve padaliniai saugomi normalizuotai.
*/
subtype t_padaliniai is rdm_utl.t_string_array; --is table of number index by pls_integer;

/*
  Autorizavimo kontekstas
*/
type t_context is record(
  naudotojo_id varchar2(20),
  naudotojo_vardas varchar2(250),
  el_pastas varchar2(250),
  institucijos_id varchar2(20),
  institucijos_pavad varchar2(100),
  padaliniai t_padaliniai,
  roles t_roles,
  vardas varchar2(100),
  pavarde varchar2(100),
  tipas varchar2(100));


/*
  Autentifikavimo funkcija, kurią kviečia APEX autentifikavimo schema.
  %param p_username Vartotojo vardas. Privalomas parametras dėl APEX reikalavimų.
  %param p_password Vartotojo slaptažodis. Privalomas parametras dėl APEX reireikalavimų.
  %return TRUE arba FALSE. Kadangi naudojamas SSO sprendimas, f-ja visada grąžina TRUE reikšmę.
*/
function login(
  p_username in varchar2,
  p_password in varchar2) return boolean;

/*
  Procedūra atjungia sistemos naudotoją.
*/
procedure logout;

function check_min(
  p_eppn in varchar2,
  p_affiliation in varchar2) return number;

function ins(
  p_min_vardas in varchar2,
  p_min_tipas in varchar2) return number;

function get_min_type(
  p_affiliation in varchar2) return varchar2;

procedure check_istaiga(
  p_org_kodas in varchar2,
  p_min_id in number);

procedure check_istaiga_existance(
  p_org_kodas in varchar2);

/*
  Funkcija transformuoja Shibboleth atributus sistemos autorizavimo poreikiams:
  1. Užpildo naudotojo id.
  2. Užpildo instituciją, su kuria susijęs naudotojas.
  3. Užpildo institucijos padalinius, su kuriais susijęs naudotojas.
  4. Užpildo naudotojo turimas roles.
  %param p_shibboloeth_attributes Shibboleth atributai.
  %return Autorizavimo kontekstas.
*/
function get_context(
  p_shibboleth_attributes shibboleth.t_attributes) return t_context;

/*
  UNIT tests
*/
procedure test_get_tutor_context;

function test_set_tutor_context return t_context;

function test_set_stud_context return t_context;

end rdm_auth;

/
--------------------------------------------------------
--  DDL for Package RDM_JOB_PROCEDURES
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_JOB_PROCEDURES" is

/*
  Rašto darbų mainavietės periodiniai foniniai darbai ir paprastas API jų valdymui.
  Visi rašto darbų mainavietės foniniai darbai turi būti keliami į šį paketą.
  
  Periodiniai foniniai darbai:
  1. Dokumento originalumo patikrinimas EPAS sistemoje - send_docs_to_epas.
  2. Dokumento panašuo procento gavimas iš EPAS - set_similarity_score.
  
  Foninių darbų API:
  1. Įjungimas - enable_job.
  2. Išjungimas - disable_job.
  3. Periodiškumo keitimas - alter_job.
*/

/*-----------------------------------------------------
  Periodiniai foniniai rašto darbų mainavietės darbai
-----------------------------------------------------*/

/*
  Periodinis įkeltų dokumentų originalumo patikrinimas EPAS sistemoje. Visi 
  dokumentai su būsena "Įkeltas" siunčiami į EPAS originalumui patikrinti,
  kviečiama rdm_ws.get_oa_id.
*/
procedure send_docs_to_epas;

/*
  Periodinis dokumento panašumo procento gavimas iš EPAS. Dokumentams, kurie jau
  patikrinti EPAS sistemoje (užpildytas originalumo ataskaitos identifikatorius),
  tačiau neturi panašumo procento, kreipiamasi į EPAS WS, panašumo procentui
  gauti, kviečiama rdm_ws.get_sim_score.
  
  Pakeitimai:
  2014-06-09
  Tomas Maroščikas
  https://www.asseco.lt/redmine/issues/30898
  Padaryta, kad rašto darbų panašumo procentas būtų nuolatos atnaujinamas.
  Iki šiol buvo užpildomas vieną kartą ir daugiau nebeatnaujinamas, tačiau
  iš veiklos pusės tai negerai, nes jis gali pasikeisti.
*/
procedure set_similarity_score;


/*--------------------------------------------
  Foninių darbų API
----------------------------------------------*/

/*
  Foninio darbo įjungimas.
  %param p_job_name Foninio darbo pavadinimas.
*/
procedure enable_job(
  p_job_name in varchar2);


/*
  Foninio darbo išjungimas.
  %param p_job_name Foninio darbo pavadinimas.
*/
procedure disable_job(
  p_job_name in varchar2);

/*
  Foninio darbo periodiškumo keitimas.
  Procedūra pakeičia Oracle SCHEDULER darbo repeat_scheduler savybės FREQ ir
  INTERVAL dalis, todėl reikia naudoti atitinkamą SCHEDULER sintaksę.
  %param p_name Foninio darbo pavadinimas.
  %param p_value INTERVAL dalis, pvz. 1.
  %param p_freq FREQ dalis, pvz. YEARLY, MONTHLY, WEEKLY, DAILY, HOURLY, 
  MINUTELY, SECONDLY.
*/
procedure alter_job(
  p_name in varchar2,
  p_value in number,
  p_freq in varchar2);

end RDM_JOB_PROCEDURES;

/
--------------------------------------------------------
--  DDL for Package RDM_LYG
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_LYG" is
/*
  Lentelės RDM_LYG (RDM_LYGMENYS) API paketas
*/
/*
  Konstantos
*/
TABLE_NAME      constant varchar2(30) := 'RDM_LYG';
LYG_ID          constant varchar2(30) := 'LYG_ID';
LYG_PAVADINIMAS constant varchar2(30) := 'LYG_PAVADINIMAS';
LYG_ORG_ID      constant varchar2(30) := 'LYG_ORG_ID';
LYG_KODAS       constant varchar2(30) := 'LYG_KODAS';
LYG_ADRESAS     constant varchar2(30) := 'LYG_ADRESAS';

/*
  Tipai
*/
type t_ind is record(
  lyg_id          boolean default false,
  lyg_pavadinimas boolean default false,
  lyg_org_id      boolean default false,
  lyg_kodas       boolean default false,
  lyg_adresas     boolean default false);

type t_val is record(
  lyg_id          number(10),
  lyg_pavadinimas varchar2(240),
  lyg_org_id      number(10),
  lyg_kodas       varchar2(20),
  lyg_adresas     varchar2(240));

type t_rec is record(
  i t_ind,
  v t_val);

/*
  Paprogrami� prototipai
*/

procedure check_delete(
  p_id in number);

procedure del(
  p_id in number,
  p_raise_error in boolean default true);

function find_uk(
  p_kodas in varchar2,
  p_org_id in number,
  p_raise_error in boolean default true) return rdm_lyg.t_val;

function get_lyg_id return number;
pragma restrict_references(get_lyg_id, wnds, wnps);

procedure ins(
  p_rec in out nocopy t_rec);

function sel(
  p_id in number,
  p_raise_error in boolean default true) return t_rec;
pragma restrict_references(sel, wnds, wnps);

procedure upd(
  p_id in number,
  p_rec in out nocopy t_rec,
  p_raise_error in boolean default true);

procedure synchronize(
  p_id in number,
  p_org_id in number);

end;

/
--------------------------------------------------------
--  DDL for Package RDM_MIN
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_MIN" is
/*
  Lentelės rdm_nariai (RDM_NARIAI) API paketas
*/
/*
  Konstantos
*/
TABLE_NAME    constant varchar2(30) := 'RDM_NARIAI';
MIN_ID        constant varchar2(30) := 'MIN_ID';
MIN_TIPAS     constant varchar2(30) := 'MIN_TIPAS';
MIN_EL_PASTAS constant varchar2(30) := 'MIN_EL_PASTAS';
MIN_VARDAS    constant varchar2(30) := 'MIN_VARDAS';
MIN_ORG_ID    constant varchar2(30) := 'MIN_ORG_ID';
MIN_PAVARDE   constant varchar2(30) := 'MIN_PAVARDE';
MIN_LYG_ID    constant varchar2(30) := 'MIN_LYG_ID';

/*
  Tipai
*/
type t_ind is record(
  min_id        boolean default false,
  min_tipas     boolean default false,
  min_el_pastas boolean default false,
  min_vardas    boolean default false,
  min_org_id    boolean default false,
  min_pavarde   boolean default false,
  min_lyg_id    boolean default false);

type t_val is record(
  min_id        number(10),
  min_tipas     varchar2(1),
  min_el_pastas varchar2(240),
  min_vardas    varchar2(240),
  min_org_id    number(10),
  min_pavarde   varchar2(240),
  min_lyg_id    number(10));

type t_rec is record(
  i t_ind,
  v t_val);

/*
  Paprogrami� prototipai
*/

procedure check_delete(
  p_id in number);

procedure del(
  p_id in number,
  p_raise_error in boolean default true);

function find_uk(
  p_el_pastas in varchar2,
  p_raise_error in boolean default true) return rdm_min.t_val;

function get_min_id return number;
pragma restrict_references(get_min_id, wnds, wnps);

procedure ins(
  p_rec in out nocopy t_rec);

function sel(
  p_id in number,
  p_raise_error in boolean default true) return t_rec;
pragma restrict_references(sel, wnds, wnps);

procedure upd(
  p_id in number,
  p_rec in out nocopy t_rec,
  p_raise_error in boolean default true);

procedure synchronize(
  p_id in number,
  p_email in varchar2,
  p_org_id in number,
  p_givenname in varchar2,
  p_surname in varchar2,
  p_affiliation in varchar2);

end;

/
--------------------------------------------------------
--  DDL for Package RDM_ORG
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_ORG" is
/*
  Lentelės RDM_ORG (RDM_ORGANIZACIJOS) API paketas
*/
/*
  Konstantos
*/
TABLE_NAME      constant varchar2(30) := 'RDM_ORG';
ORG_ID          constant varchar2(30) := 'ORG_ID';
ORG_PAVADINIMAS constant varchar2(30) := 'ORG_PAVADINIMAS';
ORG_KODAS       constant varchar2(30) := 'ORG_KODAS';
ORG_ADRESAS     constant varchar2(30) := 'ORG_ADRESAS';

/*
  Tipai
*/
type t_ind is record(
  org_id          boolean default false,
  org_pavadinimas boolean default false,
  org_kodas       boolean default false,
  org_adresas     boolean default false);

type t_val is record(
  org_id          number(10),
  org_pavadinimas varchar2(240),
  org_kodas       varchar2(20),
  org_adresas     varchar2(240));

type t_rec is record(
  i t_ind,
  v t_val);

/*
  Paprogrami� prototipai
*/

procedure check_delete(
  p_id in number);

procedure del(
  p_id in number,
  p_raise_error in boolean default true);

function find_uk(
  p_kodas in varchar2,
  p_raise_error in boolean default true) return rdm_org.t_val;

function get_org_id return number;
pragma restrict_references(get_org_id, wnds, wnps);

procedure ins(
  p_rec in out nocopy t_rec);

function sel(
  p_id in number,
  p_raise_error in boolean default true) return t_rec;
pragma restrict_references(sel, wnds, wnps);

procedure upd(
  p_id in number,
  p_rec in out nocopy t_rec,
  p_raise_error in boolean default true);

procedure synchronize(
  p_kodas in varchar2);

procedure synchronize(
  p_id in number,
  p_pavadinimas in varchar2);

end;

/
--------------------------------------------------------
--  DDL for Package RDM_PRM
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_PRM" is


/*
  Lentelės RDM_PARAMETRAI API paketas
*/

/*
  Tipai
*/
subtype t_val is rdm_parametrai%rowtype;

/*
  Paprogramių prototipai
*/
function get_value(
  p_name in varchar2,
  p_default_value in varchar2 default null) return varchar2;

procedure ins(
  p_prm_pavadinimas in varchar2,
  p_prm_verte in varchar2,
  p_prm_aprasymas in varchar2 default null);

end RDM_PRM;

/
--------------------------------------------------------
--  DDL for Package RDM_RDB
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_RDB" is

/*
  Lentelės RDM_DARBAI API
*/


/*Tipų deklaracijos*/
type t_rec is record(
  rdb_id number,
  rdb_darbo_pavadinimas varchar2(240),
  rdb_metai number,
  rdb_failo_pavadinimas varchar2(240),
  rdb_ikelimo_data date,
  rdb_min_id_stud number,
  rdb_min_id_dest number,
  rdb_min_id_rec number,
  rdb_min_id_opp number,
  rdb_org_id number,
  rdb_lyg_id number,
  rdb_statusas char(1),
  rdb_turinys blob,
  rdb_mime_tipas varchar2(240),
  rdb_epas_oa_id number,
  rdb_panasumo_procentas number,
  rdb_nebetikrinti_procento number,
  rdb_pakeitimo_data date);

/*Procedūrų ir funkcijų deklaracijos*/
procedure get_content(
  p_rdb_id in number);

function blobtoclob(
  p_blob in blob) return clob;

function clobtoblob(
  p_clob in clob) return blob;

procedure add(
  p_rec in t_rec);

procedure upd(
  p_rec in t_rec);

procedure del(
  p_rdb_id in number);

procedure set_sim_score(
  p_rdb_id in number,
  p_sim_score in number,
  p_nebetikrinti_procento in number default 1);

function get_rgb(
  p_similarity_score in number) return varchar2;

end RDM_RDB;

/
--------------------------------------------------------
--  DDL for Package RDM_UTL
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_UTL" as

/*
  Paketas grupuoja bendrus sistemos ribose naudojamus tipus, konstantas, taip
  pat naudingas konvertavimo paprogrames.
*/

/*
  Konstantos
*/
C_SEMICOLON constant varchar2(1) := ';';

/*
  Tipai
*/
type t_string_array is table of varchar2(100) index by pls_integer;


/*
  Funkcija teksto eilutę išskaido į atskiras eilutes pagal nurodytą skyriklį
  %p_list% Tekstas
  %p_delimiter% Skyriklis. Pagal nutylėjimą naudojamas kabliataškis
  %return% Eilu� masyvas
*/
function split(
  p_list varchar2,
  p_delimiter varchar2 default C_SEMICOLON) return t_string_array;

end rdm_utl;

/
--------------------------------------------------------
--  DDL for Package RDM_VERSION
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_VERSION" is

/*
  Paketas skirtas idiegtų versijų saugojimui <BR>
  Autorius: Saulius Meškelė <BR>
  Data: 2012-02-11
*/

/*
  Procedūra patikrina DB bazės objektų diegimo versiją
  %param p_application Aplikacijos trumpinys.
  %param p_version Dabartinė aplikacijos versija.
  %param p_next_version Diegiama aplikacijos versija.
  %param p_default Versijos reikšmė pagal nutylėjimą.
*/
procedure check_version(
  p_application in varchar2,
  p_version in varchar2,
  p_next_version in varchar2,
  p_default in varchar2 default '0.0');

/*
  Procedūra nustato aplikacijos versiją.
  %param p_application Aplikacijos trumpinys.
  %param p_version Dabartinė aplikacijos versija.
  %param p_base_version
*/
procedure set_version(
  p_application in varchar2,
  p_version in varchar2,
  p_base_version in varchar2 default null);

end RDM_VERSION;

/
--------------------------------------------------------
--  DDL for Package RDM_WS
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "RDM_WS" is

/*
  Paketas grupuoja funkcijas ir procedūras, kurios įgyvendina rašto darbų mainavietės
  WS ir kreipinius į kitų sistemų web servisus.
  
  Šiuo metu remiantis EPAS sąsajų specifikacija rašto darbų mainavietė įgyvendina
  tokius WS:
  1. WS, kuris grąžina rašto darbų sąrašą. F-ja get_list_of_documents.
  2. WS, kuris grąžina dokumentus skirtus saugoti EPAS nuolatinio saugojimo
     rašto darbų atspaudų DB. F-ja format_full_list_documents.
  
  Šiuo metu įgyvendinti tokie kreipiniai į kitų sistemų WS:
  1. Kreipinys į EPAS WS, kuris patikrina rašto darbo originaluma. EPAS WS
     rezultatas - originalumo ataskaitos Id. F-ja get_oa_id.
  2. Kreipinys į EPAS WS, kuris grąžina nurodytos originalumo ataskaitos būseną.
     F-ja get_oa_status.
  3. Kreipinys į EPAS WS, kuris grąžina rašto darbo panašumo procentą. F-ja get_sim_score.
*/

/*
  Tipai
*/
type t_val is record(
  id number,
  file_name varchar2(4000),
  title varchar2(4000),
  student varchar2(4000),
  student_id number,
  teacher varchar2(4000),
  teacher_id number,
  reviewer varchar2(4000),
  reviewer_id number,
  opponent varchar2(4000),
  opponent_id number,
  division varchar2(4000),
  division_id number,
  organization varchar2(4000),
  organization_id number,
  year number,
  status varchar2(1),
  source_code varchar2(50));

type list_tab is table of t_val;

type t_similarity_score_ws is record(
  similarity_score number,
  report_status number,
  report_text varchar2(240));

/*-------------------------------------------------
  Rašto darbų mainavietės web servisai
--------------------------------------------------*/

/*
  Pateikti rašto darbų sąrašą
*/
function get_list_of_documents(
  p_start_date in varchar2) return list_tab pipelined;

/*
  Pateikti dokumentus skirtus saugoti EPAS nuolatinio saugojimo
  rašto darbų atspaudų DB.
*/
function format_full_list_documents(
  p_start_date in varchar2) return clob;


/*-------------------------------------------------
  Kreipiniai į EPAS web servisus
--------------------------------------------------*/

/*
  Patikrinti rašto darbo originalumą
*/
procedure get_oa_id(
  p_doc_id in number);

/*
  Gauti originalumo ataskaitos būseną
*/
function get_oa_status(
  p_oa_id in number) return varchar2;

/*
  Gauti rašto darbo panašumo procentą
*/
function get_sim_score(
  p_oa_id in number) return t_similarity_score_ws;

end RDM_WS;

/
--------------------------------------------------------
--  DDL for Package SHIBBOLETH
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE "SHIBBOLETH" as

/*
  Sąsaja tarp Shibboleth ir galutinės sistemos, šiuo atveju rašto darbų mainavietės. Paketas
  grupuoja su Shibboleth susijusią logiką ir ją nukrauna nuo galutinės sistemos.
  Paketas yra gana bendras (nepririštas prie konkrečios IS), todėl gali būti naudojamas ir
  kitose IS, kur SSO sprendimas paremtas Shibboleth programine įranga. Vienintelis dalykas
  ką reikia pasikeisti, tai susižiūrėti, kokie atributai atkeliauja iš Shibboleth, žr. tipą
  t_attributes.
*/

/*
  Shibboleth naudojamų atributų aibė. Naudojami atributai aprašomi Shibboleth konfigūraciniame
  faile attribute-map.xml. Failą papildžius naujais atributais, reikia papildyti šią struktūrą,
  taip pat atributus užpildančią f-ją get_attributes.
*/
type t_attributes is record(
  uid varchar2(100),                                    /* pvz., 211950810/767 */
  commonname varchar2(250),                             /* pvz., Vardas Pavarde */
  surname varchar2(100),                                /* pvz., Pavarde */
  givenname varchar2(100),                              /* pvz., Vardas */
  displayname varchar2(250),                            /* pvz., Vardas Pavarde */
  email varchar2(250),                                  /* pvz., vu.admin@asseco.lt */
  schachomeorganizationtype varchar2(1000),     /* pvz., urn:mace:terena.org:schac:homeOrganizationType:lt:university */
  schachomeorganization varchar2(100),          /* pvz., 211950810 */
  edupersonprincipalname varchar2(250),         /* pvz., vu.admin@asseco.lt */
  edupersonaffiliation varchar2(100),           /* pvz., employee */
  edupersontargetedid varchar2(100),            /* pvz., 767 */
  elabainstitutionid varchar2(100),             /* Naudotojo institucijos identifikatorius. Visada vienas, pvz., 17 */
  elabaprivileges varchar2(32000),     /* Rolės. eLABa perduoda visas turimas roles, tačiau mums įdomios tik susijusios su EPAS:
                                            1. DOCUMENT__SUPERVISOR__EDIT_EPAS -  Darbo vadovui tvarkyti priskirtų dokumentų EPAS duomenis
                                            2. DOCUMENT__REVIEWER__EDIT_EPAS - Recenzentui tvarkyti priskirtų dokumentų EPAS duomenis
                                            3. DOCUMENT__DEPARTMENT__EDIT_EPAS - Darbuotojui tvarkyti savo padalinio dokumentų EPAS duomenis
                                            4. DOCUMENT__INSTITUTION__EDIT_EPAS - Darbuotojui tvarkyti savo institucijos dokumentų EPAS duomenis
                                        */
  elabadutydepartments varchar2(1000)); /* Naudotojo padaliniai, gali būti keli, atsikirti kabliataškiu (;). Pvz., 119;117;115;123;121 */

/*
  Funkcija užpildo ir grąžina Shibboleth perduotų atributų reikšmes. Į DB atributai
  atkeliauja kaip CGI aplinkos kintamieji.
  %return t_shibboleth_attributes tipo kintamasis užpildytas iš SSO gautomis atributų reikšmėmis.
*/
function get_attributes return t_attributes;

/*
  UNIT tests
*/

/* Gauti dėstytojo atributus */
function test_get_tutor_attributes return t_attributes;

/* Gauti studento atributus */
function test_get_stud_attributes return t_attributes;

end shibboleth;

/
