--------------------------------------------------------
--  File created - Sekmadienis-balandis-12-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package Body HTTP
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "HTTP" as

procedure set_req_headers(
  p_headers in t_headers,
  p_request in out utl_http.req) is
  
  l_index varchar2(200);
begin
  l_index := p_headers.first;
  while l_index is not null loop
    utl_http.set_header(p_request, l_index, p_headers(l_index));
    plog.debug('Setting HTTP header '||l_index||':'||p_headers(l_index));
    l_index := p_headers.next(l_index);
  end loop;
end;

procedure test_url(
  p_url in varchar2,
  p_request_headers in t_headers) is
  
  l_http_request utl_http.req;
  l_http_response utl_http.resp;
  l_http_resp_header_name varchar2(100);
  l_http_resp_header_value varchar2(100);
begin
  plog.debug('Testing HTTP connection to '||p_url);
  l_http_request := utl_http.begin_request(p_url, 'GET', utl_http.HTTP_VERSION_1_1);
  set_req_headers(p_request_headers, l_http_request);
  l_http_response := utl_http.get_response(l_http_request);
  dbms_output.put_line('Response code:'||l_http_response.status_code);
  plog.debug('Response code:'||l_http_response.status_code);
  for i in 1..utl_http.get_header_count(l_http_response) loop
	  utl_http.get_header(l_http_response, i, l_http_resp_header_name, l_http_resp_header_value);
    dbms_output.put_line(l_http_resp_header_name||':'||l_http_resp_header_value);
    plog.debug('Response header '||l_http_resp_header_name||':'||l_http_resp_header_value);
  end loop;
  utl_http.end_response(l_http_response);
end;

function do_get_request(
    p_url in varchar2,
    p_request_headers in t_headers) return varchar2 as
  begin
    -- TODO: Implementation required for function HTTP.get_request
    return null;
  end do_get_request;

function do_post_request(
    p_url in varchar2,
    p_request_headers in t_headers,
    p_request_body in varchar2) return varchar2 as
  
  l_http_request utl_http.req;
  l_http_response utl_http.resp;
  l_http_response_content varchar2(32767);
BEGIN
  l_http_request := utl_http.begin_request(p_url, 'POST', utl_http.HTTP_VERSION_1_1);
  set_req_headers(p_request_headers, l_http_request);
  utl_http.write_text(l_http_request, p_request_body);
  l_http_response := utl_http.get_response(l_http_request);
  utl_http.read_text(l_http_response, l_http_response_content);
  utl_http.end_response(l_http_response);
  
  return l_http_response_content;
end do_post_request;

function do_post_request(
  p_url in varchar2,
  p_request_headers in t_headers,
  p_request_body in clob) return clob as
  
  l_content_length pls_integer;
  l_buffer varchar2(32767);
  l_offset pls_integer := 1;
  l_amount pls_integer := 2000;
  
  l_http_request utl_http.req;
  l_http_response utl_http.resp;
  l_http_response_content clob;
begin
  /* Request */
  plog.debug('HTTP reguest to:'||p_url);
  l_http_request := utl_http.begin_request(p_url, 'POST', utl_http.HTTP_VERSION_1_1);
  l_content_length := dbms_lob.getlength(p_request_body);
  if l_content_length > 32767 then
    utl_http.set_header(l_http_request, 'Transfer-Encoding', 'chunked');
    plog.debug('Setting HTTP header Transfer-Encoding:chunked');
  end if;
  set_req_headers(p_request_headers, l_http_request);
  while(l_offset < l_content_length) loop
    dbms_lob.read(p_request_body, l_amount, l_offset, l_buffer);
    utl_http.write_text(l_http_request, l_buffer);
    l_offset := l_offset + l_amount;
  end loop;
  
  /* Response */
  l_http_response := utl_http.get_response(l_http_request);
  
  dbms_lob.createtemporary(l_http_response_content, false);
  begin
    loop
      utl_http.read_text(l_http_response, l_buffer, null);
      dbms_lob.writeappend(l_http_response_content, length(l_buffer), l_buffer);
    end loop;
  exception 
    when utl_http.end_of_body then
      utl_http.end_response(l_http_response);
  end;
  return l_http_response_content;
end do_post_request;
  
function do_put_request(
  p_url in varchar2,
  p_request_headers in t_headers,
  p_request_body in varchar2) return varchar2 as
  
  l_http_request utl_http.req;
  l_http_response utl_http.resp;
  l_http_response_content varchar2(32767);
BEGIN
  l_http_request := utl_http.begin_request(p_url, 'PUT', utl_http.HTTP_VERSION_1_1);
  set_req_headers(p_request_headers, l_http_request);
  utl_http.write_text(l_http_request, p_request_body);
  l_http_response := utl_http.get_response(l_http_request);
  utl_http.read_text(l_http_response, l_http_response_content);
  utl_http.end_response(l_http_response);
  
  return l_http_response_content;
end do_put_request;

end http;

/
--------------------------------------------------------
--  DDL for Package Body JSON_AC
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "JSON_AC" as
  procedure object_remove(p_self in out nocopy json, pair_name varchar2) as
  begin p_self.remove(pair_name); end;
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value json_value, position pls_integer default null) as
  begin p_self.put(pair_name, pair_value, position); end;
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value varchar2, position pls_integer default null) as
  begin p_self.put(pair_name, pair_value, position); end;
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value number, position pls_integer default null) as
  begin p_self.put(pair_name, pair_value, position); end;
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value boolean, position pls_integer default null) as
  begin p_self.put(pair_name, pair_value, position); end;
  procedure object_check_duplicate(p_self in out nocopy json, v_set boolean) as
  begin p_self.check_duplicate(v_set); end;
  procedure object_remove_duplicates(p_self in out nocopy json) as
  begin p_self.remove_duplicates; end;

  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value json, position pls_integer default null) as
  begin p_self.put(pair_name, pair_value, position); end;
  procedure object_put(p_self in out nocopy json, pair_name varchar2, pair_value json_list, position pls_integer default null) as
  begin p_self.put(pair_name, pair_value, position); end;

  function object_count(p_self in json) return number as
  begin return p_self.count; end;
  function object_get(p_self in json, pair_name varchar2) return json_value as
  begin return p_self.get(pair_name); end;
  function object_get(p_self in json, position pls_integer) return json_value as
  begin return p_self.get(position); end;
  function object_index_of(p_self in json, pair_name varchar2) return number as
  begin return p_self.index_of(pair_name); end;
  function object_exist(p_self in json, pair_name varchar2) return boolean as
  begin return p_self.exist(pair_name); end;

  function object_to_char(p_self in json, spaces boolean default true, chars_per_line number default 0) return varchar2 as
  begin return p_self.to_char(spaces, chars_per_line); end;
  procedure object_to_clob(p_self in json, buf in out nocopy clob, spaces boolean default false, chars_per_line number default 0, erase_clob boolean default true) as
  begin p_self.to_clob(buf, spaces, chars_per_line, erase_clob); end;
  procedure object_print(p_self in json, spaces boolean default true, chars_per_line number default 8192, jsonp varchar2 default null) as
  begin p_self.print(spaces, chars_per_line, jsonp); end;
  procedure object_htp(p_self in json, spaces boolean default false, chars_per_line number default 0, jsonp varchar2 default null) as
  begin p_self.htp(spaces, chars_per_line, jsonp); end;

  function object_to_json_value(p_self in json) return json_value as
  begin return p_self.to_json_value; end;
  function object_path(p_self in json, json_path varchar2, base number default 1) return json_value as
  begin return p_self.path(json_path, base); end;

  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem json_value, base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem varchar2  , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem number    , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem boolean   , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem json_list , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure object_path_put(p_self in out nocopy json, json_path varchar2, elem json      , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;

  procedure object_path_remove(p_self in out nocopy json, json_path varchar2, base number default 1) as
  begin p_self.path_remove(json_path, base); end;

  function object_get_values(p_self in json) return json_list as
  begin return p_self.get_values; end;
  function object_get_keys(p_self in json) return json_list as
  begin return p_self.get_keys; end;

  --json_list type
  procedure array_append(p_self in out nocopy json_list, elem json_value, position pls_integer default null) as
  begin p_self.append(elem, position); end;
  procedure array_append(p_self in out nocopy json_list, elem varchar2, position pls_integer default null) as
  begin p_self.append(elem, position); end;
  procedure array_append(p_self in out nocopy json_list, elem number, position pls_integer default null) as
  begin p_self.append(elem, position); end;
  procedure array_append(p_self in out nocopy json_list, elem boolean, position pls_integer default null) as
  begin p_self.append(elem, position); end;
  procedure array_append(p_self in out nocopy json_list, elem json_list, position pls_integer default null) as
  begin p_self.append(elem, position); end;

  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem json_value) as
  begin p_self.replace(position, elem); end;
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem varchar2) as
  begin p_self.replace(position, elem); end;
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem number) as
  begin p_self.replace(position, elem); end;
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem boolean) as
  begin p_self.replace(position, elem); end;
  procedure array_replace(p_self in out nocopy json_list, position pls_integer, elem json_list) as
  begin p_self.replace(position, elem); end;

  function array_count(p_self in json_list) return number as
  begin return p_self.count; end;
  procedure array_remove(p_self in out nocopy json_list, position pls_integer) as
  begin p_self.remove(position); end;
  procedure array_remove_first(p_self in out nocopy json_list) as
  begin p_self.remove_first; end;
  procedure array_remove_last(p_self in out nocopy json_list) as
  begin p_self.remove_last; end;
  function array_get(p_self in json_list, position pls_integer) return json_value as
  begin return p_self.get(position); end;
  function array_head(p_self in json_list) return json_value as
  begin return p_self.head; end;
  function array_last(p_self in json_list) return json_value as
  begin return p_self.last; end;
  function array_tail(p_self in json_list) return json_list as
  begin return p_self.tail; end;

  function array_to_char(p_self in json_list, spaces boolean default true, chars_per_line number default 0) return varchar2 as
  begin return p_self.to_char(spaces, chars_per_line); end;
  procedure array_to_clob(p_self in json_list, buf in out nocopy clob, spaces boolean default false, chars_per_line number default 0, erase_clob boolean default true) as
  begin p_self.to_clob(buf, spaces, chars_per_line, erase_clob); end;
  procedure array_print(p_self in json_list, spaces boolean default true, chars_per_line number default 8192, jsonp varchar2 default null) as
  begin p_self.print(spaces, chars_per_line, jsonp); end;
  procedure array_htp(p_self in json_list, spaces boolean default false, chars_per_line number default 0, jsonp varchar2 default null) as
  begin p_self.htp(spaces, chars_per_line, jsonp); end;

  function array_path(p_self in json_list, json_path varchar2, base number default 1) return json_value as
  begin return p_self.path(json_path, base); end;
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem json_value, base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem varchar2  , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem number    , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem boolean   , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;
  procedure array_path_put(p_self in out nocopy json_list, json_path varchar2, elem json_list , base number default 1) as
  begin p_self.path_put(json_path, elem, base); end;

  procedure array_path_remove(p_self in out nocopy json_list, json_path varchar2, base number default 1) as
  begin p_self.path_remove(json_path, base); end;

  function array_to_json_value(p_self in json_list) return json_value as
  begin return p_self.to_json_value; end;

  --json_value


  function jv_get_type(p_self in json_value) return varchar2 as
  begin return p_self.get_type; end;
  function jv_get_string(p_self in json_value, max_byte_size number default null, max_char_size number default null) return varchar2 as
  begin return p_self.get_string(max_byte_size, max_char_size); end;
  procedure jv_get_string(p_self in json_value, buf in out nocopy clob) as
  begin p_self.get_string(buf); end;
  function jv_get_number(p_self in json_value) return number as
  begin return p_self.get_number; end;
  function jv_get_bool(p_self in json_value) return boolean as
  begin return p_self.get_bool; end;
  function jv_get_null(p_self in json_value) return varchar2 as
  begin return p_self.get_null; end;

  function jv_is_object(p_self in json_value) return boolean as
  begin return p_self.is_object; end;
  function jv_is_array(p_self in json_value) return boolean as
  begin return p_self.is_array; end;
  function jv_is_string(p_self in json_value) return boolean as
  begin return p_self.is_string; end;
  function jv_is_number(p_self in json_value) return boolean as
  begin return p_self.is_number; end;
  function jv_is_bool(p_self in json_value) return boolean as
  begin return p_self.is_bool; end;
  function jv_is_null(p_self in json_value) return boolean as
  begin return p_self.is_null; end;

  function jv_to_char(p_self in json_value, spaces boolean default true, chars_per_line number default 0) return varchar2 as
  begin return p_self.to_char(spaces, chars_per_line); end;
  procedure jv_to_clob(p_self in json_value, buf in out nocopy clob, spaces boolean default false, chars_per_line number default 0, erase_clob boolean default true) as
  begin p_self.to_clob(buf, spaces, chars_per_line, erase_clob); end;
  procedure jv_print(p_self in json_value, spaces boolean default true, chars_per_line number default 8192, jsonp varchar2 default null) as
  begin p_self.print(spaces, chars_per_line, jsonp); end;
  procedure jv_htp(p_self in json_value, spaces boolean default false, chars_per_line number default 0, jsonp varchar2 default null) as
  begin p_self.htp(spaces, chars_per_line, jsonp); end;

  function jv_value_of(p_self in json_value, max_byte_size number default null, max_char_size number default null) return varchar2 as
  begin return p_self.value_of(max_byte_size, max_char_size); end;

end json_ac;

/
--------------------------------------------------------
--  DDL for Package Body JSON_EXT
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "JSON_EXT" as
  scanner_exception exception;
  pragma exception_init(scanner_exception, -20100);
  parser_exception exception;
  pragma exception_init(parser_exception, -20101);
  jext_exception exception;
  pragma exception_init(jext_exception, -20110);

  --extra function checks if number has no fraction
  function is_integer(v json_value) return boolean as
    myint number(38); --the oracle way to specify an integer
  begin
    if(v.is_number) then
      myint := v.get_number;
      return (myint = v.get_number); --no rounding errors?
    else
      return false;
    end if;
  end;

  --extension enables json to store dates without comprimising the implementation
  function to_json_value(d date) return json_value as
  begin
    return json_value(to_char(d, format_string));
  end;

  --notice that a date type in json is also a varchar2
  function is_date(v json_value) return boolean as
    temp date;
  begin
    temp := json_ext.to_date2(v);
    return true;
  exception
    when others then
      return false;
  end;

  --convertion is needed to extract dates
  function to_date2(v json_value) return date as
  begin
    if(v.is_string) then
      return to_date(v.get_string, format_string);
    else
      raise_application_error(-20110, 'Anydata did not contain a date-value');
    end if;
  exception
    when others then
      raise_application_error(-20110, 'Anydata did not contain a date on the format: '||format_string);
  end;

  --Json Path parser
  function parsePath(json_path varchar2, base number default 1) return json_list as
    build_path varchar2(32767) := '[';
    buf varchar2(4);
    endstring varchar2(1);
    indx number := 1;
    ret json_list;

    procedure next_char as
    begin
      if(indx <= length(json_path)) then
        buf := substr(json_path, indx, 1);
        indx := indx + 1;
      else
        buf := null;
      end if;
    end;
    --skip ws
    procedure skipws as begin while(buf in (chr(9),chr(10),chr(13),' ')) loop next_char; end loop; end;

  begin
    next_char();
    while(buf is not null) loop
      if(buf = '.') then
        next_char();
        if(buf is null) then raise_application_error(-20110, 'JSON Path parse error: . is not a valid json_path end'); end if;
        if(not regexp_like(buf, '^[[:alnum:]\_ ]+', 'c') ) then
          raise_application_error(-20110, 'JSON Path parse error: alpha-numeric character or space expected at position '||indx);
        end if;

        if(build_path != '[') then build_path := build_path || ','; end if;
        build_path := build_path || '"';
        while(regexp_like(buf, '^[[:alnum:]\_ ]+', 'c') ) loop
          build_path := build_path || buf;
          next_char();
        end loop;
        build_path := build_path || '"';
      elsif(buf = '[') then
        next_char();
        skipws();
        if(buf is null) then raise_application_error(-20110, 'JSON Path parse error: [ is not a valid json_path end'); end if;
        if(buf in ('1','2','3','4','5','6','7','8','9') or (buf = '0' and base = 0)) then
          if(build_path != '[') then build_path := build_path || ','; end if;
          while(buf in ('0','1','2','3','4','5','6','7','8','9')) loop
            build_path := build_path || buf;
            next_char();
          end loop;
        elsif (regexp_like(buf, '^(\"|\'')', 'c')) then
          endstring := buf;
          if(build_path != '[') then build_path := build_path || ','; end if;
          build_path := build_path || '"';
          next_char();
          if(buf is null) then raise_application_error(-20110, 'JSON Path parse error: premature json_path end'); end if;
          while(buf != endstring) loop
            build_path := build_path || buf;
            next_char();
            if(buf is null) then raise_application_error(-20110, 'JSON Path parse error: premature json_path end'); end if;
            if(buf = '\') then
              next_char();
              build_path := build_path || '\' || buf;
              next_char();
            end if;
          end loop;
          build_path := build_path || '"';
          next_char();
        else
          raise_application_error(-20110, 'JSON Path parse error: expected a string or an positive integer at '||indx);
        end if;
        skipws();
        if(buf is null) then raise_application_error(-20110, 'JSON Path parse error: premature json_path end'); end if;
        if(buf != ']') then raise_application_error(-20110, 'JSON Path parse error: no array ending found. found: '|| buf); end if;
        next_char();
        skipws();
      elsif(build_path = '[') then
        if(not regexp_like(buf, '^[[:alnum:]\_ ]+', 'c') ) then
          raise_application_error(-20110, 'JSON Path parse error: alpha-numeric character or space expected at position '||indx);
        end if;
        build_path := build_path || '"';
        while(regexp_like(buf, '^[[:alnum:]\_ ]+', 'c') ) loop
          build_path := build_path || buf;
          next_char();
        end loop;
        build_path := build_path || '"';
      else
        raise_application_error(-20110, 'JSON Path parse error: expected . or [ found '|| buf || ' at position '|| indx);
      end if;

    end loop;

    build_path := build_path || ']';
    build_path := replace(replace(replace(replace(replace(build_path, chr(9), '\t'), chr(10), '\n'), chr(13), '\f'), chr(8), '\b'), chr(14), '\r');

    ret := json_list(build_path);
    if(base != 1) then
      --fix base 0 to base 1
      declare
        elem json_value;
      begin
        for i in 1 .. ret.count loop
          elem := ret.get(i);
          if(elem.is_number) then
            ret.replace(i,elem.get_number()+1);
          end if;
        end loop;
      end;
    end if;

    return ret;
  end parsePath;

  --JSON Path getters
  function get_json_value(obj json, v_path varchar2, base number default 1) return json_value as
    path json_list;
    ret json_value;
    o json; l json_list;
  begin
    path := parsePath(v_path, base);
    ret := obj.to_json_value;
    if(path.count = 0) then return ret; end if;

    for i in 1 .. path.count loop
      if(path.get(i).is_string()) then
        --string fetch only on json
        o := json(ret);
        ret := o.get(path.get(i).get_string());
      else
        --number fetch on json and json_list
        if(ret.is_array()) then
          l := json_list(ret);
          ret := l.get(path.get(i).get_number());
        else
          o := json(ret);
          l := o.get_values();
          ret := l.get(path.get(i).get_number());
        end if;
      end if;
    end loop;

    return ret;
  exception
    when scanner_exception then raise;
    when parser_exception then raise;
    when jext_exception then raise;
    when others then return null;
  end get_json_value;

  --JSON Path getters
  function get_string(obj json, path varchar2, base number default 1) return varchar2 as
    temp json_value;
  begin
    temp := get_json_value(obj, path, base);
    if(temp is null or not temp.is_string) then
      return null;
    else
      return temp.get_string;
    end if;
  end;

  function get_number(obj json, path varchar2, base number default 1) return number as
    temp json_value;
  begin
    temp := get_json_value(obj, path, base);
    if(temp is null or not temp.is_number) then
      return null;
    else
      return temp.get_number;
    end if;
  end;

  function get_json(obj json, path varchar2, base number default 1) return json as
    temp json_value;
  begin
    temp := get_json_value(obj, path, base);
    if(temp is null or not temp.is_object) then
      return null;
    else
      return json(temp);
    end if;
  end;

  function get_json_list(obj json, path varchar2, base number default 1) return json_list as
    temp json_value;
  begin
    temp := get_json_value(obj, path, base);
    if(temp is null or not temp.is_array) then
      return null;
    else
      return json_list(temp);
    end if;
  end;

  function get_bool(obj json, path varchar2, base number default 1) return boolean as
    temp json_value;
  begin
    temp := get_json_value(obj, path, base);
    if(temp is null or not temp.is_bool) then
      return null;
    else
      return temp.get_bool;
    end if;
  end;

  function get_date(obj json, path varchar2, base number default 1) return date as
    temp json_value;
  begin
    temp := get_json_value(obj, path, base);
    if(temp is null or not is_date(temp)) then
      return null;
    else
      return json_ext.to_date2(temp);
    end if;
  end;

  /* JSON Path putter internal function */
  procedure put_internal(obj in out nocopy json, v_path varchar2, elem json_value, base number) as
    val json_value := elem;
    path json_list;
    backreference json_list := json_list();

    keyval json_value; keynum number; keystring varchar2(4000);
    temp json_value := obj.to_json_value;
    obj_temp  json;
    list_temp json_list;
    inserter json_value;
  begin
    path := json_ext.parsePath(v_path, base);
    if(path.count = 0) then raise_application_error(-20110, 'JSON_EXT put error: cannot put with empty string.'); end if;

    --build backreference
    for i in 1 .. path.count loop
      --backreference.print(false);
      keyval := path.get(i);
      if (keyval.is_number()) then
        --nummer index
        keynum := keyval.get_number();
        if((not temp.is_object()) and (not temp.is_array())) then
          if(val is null) then return; end if;
          backreference.remove_last;
          temp := json_list().to_json_value();
          backreference.append(temp);
        end if;

        if(temp.is_object()) then
          obj_temp := json(temp);
          if(obj_temp.count < keynum) then
            if(val is null) then return; end if;
            raise_application_error(-20110, 'JSON_EXT put error: access object with to few members.');
          end if;
          temp := obj_temp.get(keynum);
        else
          list_temp := json_list(temp);
          if(list_temp.count < keynum) then
            if(val is null) then return; end if;
            --raise error or quit if val is null
            for i in list_temp.count+1 .. keynum loop
              list_temp.append(json_value.makenull);
            end loop;
            backreference.remove_last;
            backreference.append(list_temp);
          end if;

          temp := list_temp.get(keynum);
        end if;
      else
        --streng index
        keystring := keyval.get_string();
        if(not temp.is_object()) then
          --backreference.print;
          if(val is null) then return; end if;
          backreference.remove_last;
          temp := json().to_json_value();
          backreference.append(temp);
          --raise_application_error(-20110, 'JSON_ext put error: trying to access a non object with a string.');
        end if;
        obj_temp := json(temp);
        temp := obj_temp.get(keystring);
      end if;

      if(temp is null) then
        if(val is null) then return; end if;
        --what to expect?
        keyval := path.get(i+1);
        if(keyval is not null and keyval.is_number()) then
          temp := json_list().to_json_value;
        else
          temp := json().to_json_value;
        end if;
      end if;
      backreference.append(temp);
    end loop;

  --  backreference.print(false);
  --  path.print(false);

    --use backreference and path together
    inserter := val;
    for i in reverse 1 .. backreference.count loop
  --    inserter.print(false);
      if( i = 1 ) then
        keyval := path.get(1);
        if(keyval.is_string()) then
          keystring := keyval.get_string();
        else
          keynum := keyval.get_number();
          declare
            t1 json_value := obj.get(keynum);
          begin
            keystring := t1.mapname;
          end;
        end if;
        if(inserter is null) then obj.remove(keystring); else obj.put(keystring, inserter); end if;
      else
        temp := backreference.get(i-1);
        if(temp.is_object()) then
          keyval := path.get(i);
          obj_temp := json(temp);
          if(keyval.is_string()) then
            keystring := keyval.get_string();
          else
            keynum := keyval.get_number();
            declare
              t1 json_value := obj_temp.get(keynum);
            begin
              keystring := t1.mapname;
            end;
          end if;
          if(inserter is null) then
            obj_temp.remove(keystring);
            if(obj_temp.count > 0) then inserter := obj_temp.to_json_value; end if;
          else
            obj_temp.put(keystring, inserter);
            inserter := obj_temp.to_json_value;
          end if;
        else
          --array only number
          keynum := path.get(i).get_number();
          list_temp := json_list(temp);
          list_temp.remove(keynum);
          if(not inserter is null) then
            list_temp.append(inserter, keynum);
            inserter := list_temp.to_json_value;
          else
            if(list_temp.count > 0) then inserter := list_temp.to_json_value; end if;
          end if;
        end if;
      end if;

    end loop;

  end put_internal;

  /* JSON Path putters */
  procedure put(obj in out nocopy json, path varchar2, elem varchar2, base number default 1) as
  begin
    put_internal(obj, path, json_value(elem), base);
  end;

  procedure put(obj in out nocopy json, path varchar2, elem number, base number default 1) as
  begin
    if(elem is null) then raise_application_error(-20110, 'Cannot put null-value'); end if;
    put_internal(obj, path, json_value(elem), base);
  end;

  procedure put(obj in out nocopy json, path varchar2, elem json, base number default 1) as
  begin
    if(elem is null) then raise_application_error(-20110, 'Cannot put null-value'); end if;
    put_internal(obj, path, elem.to_json_value, base);
  end;

  procedure put(obj in out nocopy json, path varchar2, elem json_list, base number default 1) as
  begin
    if(elem is null) then raise_application_error(-20110, 'Cannot put null-value'); end if;
    put_internal(obj, path, elem.to_json_value, base);
  end;

  procedure put(obj in out nocopy json, path varchar2, elem boolean, base number default 1) as
  begin
    if(elem is null) then raise_application_error(-20110, 'Cannot put null-value'); end if;
    put_internal(obj, path, json_value(elem), base);
  end;

  procedure put(obj in out nocopy json, path varchar2, elem json_value, base number default 1) as
  begin
    if(elem is null) then raise_application_error(-20110, 'Cannot put null-value'); end if;
    put_internal(obj, path, elem, base);
  end;

  procedure put(obj in out nocopy json, path varchar2, elem date, base number default 1) as
  begin
    if(elem is null) then raise_application_error(-20110, 'Cannot put null-value'); end if;
    put_internal(obj, path, json_ext.to_json_value(elem), base);
  end;

  procedure remove(obj in out nocopy json, path varchar2, base number default 1) as
  begin
    json_ext.put_internal(obj,path,null,base);
--    if(json_ext.get_json_value(obj,path) is not null) then
--    end if;
  end remove;

    --Pretty print with JSON Path
  function pp(obj json, v_path varchar2) return varchar2 as
    json_part json_value;
  begin
    json_part := json_ext.get_json_value(obj, v_path);
    if(json_part is null) then
      return '';
    else
      return json_printer.pretty_print_any(json_part); --escapes a possible internal string
    end if;
  end pp;

  procedure pp(obj json, v_path varchar2) as --using dbms_output.put_line
  begin
    dbms_output.put_line(pp(obj, v_path));
  end pp;

  -- spaces = false!
  procedure pp_htp(obj json, v_path varchar2) as --using htp.print
    json_part json_value;
  begin
    json_part := json_ext.get_json_value(obj, v_path);
    if(json_part is null) then htp.print; else
      htp.print(json_printer.pretty_print_any(json_part, false));
    end if;
  end pp_htp;

  function base64(binarydata blob) return json_list as
    obj json_list := json_list();
    c clob := empty_clob();
    benc blob;

    v_blob_offset NUMBER := 1;
    v_clob_offset NUMBER := 1;
    v_lang_context NUMBER := DBMS_LOB.DEFAULT_LANG_CTX;
    v_warning NUMBER;
    v_amount PLS_INTEGER;
--    temp varchar2(32767);

    FUNCTION encodeBlob2Base64(pBlobIn IN BLOB) RETURN BLOB IS
      vAmount NUMBER := 45;
      vBlobEnc BLOB := empty_blob();
      vBlobEncLen NUMBER := 0;
      vBlobInLen NUMBER := 0;
      vBuffer RAW(45);
      vOffset NUMBER := 1;
    BEGIN
--      dbms_output.put_line('Start base64 encoding.');
      vBlobInLen := dbms_lob.getlength(pBlobIn);
--      dbms_output.put_line('<BlobInLength>' || vBlobInLen);
      dbms_lob.createtemporary(vBlobEnc, TRUE);
      LOOP
        IF vOffset >= vBlobInLen THEN
          EXIT;
        END IF;
        dbms_lob.read(pBlobIn, vAmount, vOffset, vBuffer);
        BEGIN
          dbms_lob.append(vBlobEnc, utl_encode.base64_encode(vBuffer));
        EXCEPTION
          WHEN OTHERS THEN
          dbms_output.put_line('<vAmount>' || vAmount || '<vOffset>' || vOffset || '<vBuffer>' || vBuffer);
          dbms_output.put_line('ERROR IN append: ' || SQLERRM);
          RAISE;
        END;
        vOffset := vOffset + vAmount;
      END LOOP;
      vBlobEncLen := dbms_lob.getlength(vBlobEnc);
--      dbms_output.put_line('<BlobEncLength>' || vBlobEncLen);
--      dbms_output.put_line('Finshed base64 encoding.');
      RETURN vBlobEnc;
    END encodeBlob2Base64;
  begin
    benc := encodeBlob2Base64(binarydata);
    dbms_lob.createtemporary(c, TRUE);
    v_amount := DBMS_LOB.GETLENGTH(benc);
    DBMS_LOB.CONVERTTOCLOB(c, benc, v_amount, v_clob_offset, v_blob_offset, 1, v_lang_context, v_warning);

    v_amount := DBMS_LOB.GETLENGTH(c);
    v_clob_offset := 1;
    --dbms_output.put_line('V amount: '||v_amount);
    while(v_clob_offset < v_amount) loop
      --dbms_output.put_line(v_offset);
      --temp := ;
      --dbms_output.put_line('size: '||length(temp));
      obj.append(dbms_lob.SUBSTR(c, 4000,v_clob_offset));
      v_clob_offset := v_clob_offset + 4000;
    end loop;
    dbms_lob.freetemporary(benc);
    dbms_lob.freetemporary(c);
  --dbms_output.put_line(obj.count);
  --dbms_output.put_line(obj.get_last().to_char);
    return obj;

  end base64;


  function base64(l json_list) return blob as
    c clob := empty_clob();
    b blob := empty_blob();
    bret blob;

    v_blob_offset NUMBER := 1;
    v_clob_offset NUMBER := 1;
    v_lang_context NUMBER := 0; --DBMS_LOB.DEFAULT_LANG_CTX;
    v_warning NUMBER;
    v_amount PLS_INTEGER;

    FUNCTION decodeBase642Blob(pBlobIn IN BLOB) RETURN BLOB IS
      vAmount NUMBER := 256;--32;
      vBlobDec BLOB := empty_blob();
      vBlobDecLen NUMBER := 0;
      vBlobInLen NUMBER := 0;
      vBuffer RAW(256);--32);
      vOffset NUMBER := 1;
    BEGIN
--      dbms_output.put_line('Start base64 decoding.');
      vBlobInLen := dbms_lob.getlength(pBlobIn);
--      dbms_output.put_line('<BlobInLength>' || vBlobInLen);
      dbms_lob.createtemporary(vBlobDec, TRUE);
      LOOP
        IF vOffset >= vBlobInLen THEN
          EXIT;
        END IF;
        dbms_lob.read(pBlobIn, vAmount, vOffset, vBuffer);
        BEGIN
          dbms_lob.append(vBlobDec, utl_encode.base64_decode(vBuffer));
        EXCEPTION
          WHEN OTHERS THEN
          dbms_output.put_line('<vAmount>' || vAmount || '<vOffset>' || vOffset || '<vBuffer>' || vBuffer);
          dbms_output.put_line('ERROR IN append: ' || SQLERRM);
          RAISE;
        END;
        vOffset := vOffset + vAmount;
      END LOOP;
      vBlobDecLen := dbms_lob.getlength(vBlobDec);
--      dbms_output.put_line('<BlobDecLength>' || vBlobDecLen);
--      dbms_output.put_line('Finshed base64 decoding.');
      RETURN vBlobDec;
    END decodeBase642Blob;
  begin
    dbms_lob.createtemporary(c, TRUE);
    for i in 1 .. l.count loop
      dbms_lob.append(c, l.get(i).get_string());
    end loop;
    v_amount := DBMS_LOB.GETLENGTH(c);
--    dbms_output.put_line('L C'||v_amount);

    dbms_lob.createtemporary(b, TRUE);
    DBMS_LOB.CONVERTTOBLOB(b, c, dbms_lob.lobmaxsize, v_clob_offset, v_blob_offset, 1, v_lang_context, v_warning);
    dbms_lob.freetemporary(c);
    v_amount := DBMS_LOB.GETLENGTH(b);
--    dbms_output.put_line('L B'||v_amount);

    bret := decodeBase642Blob(b);
    dbms_lob.freetemporary(b);
    return bret;

  end base64;

  function encode(binarydata blob) return json_value as
    obj json_value;
    c clob := empty_clob();
    benc blob;

    v_blob_offset NUMBER := 1;
    v_clob_offset NUMBER := 1;
    v_lang_context NUMBER := DBMS_LOB.DEFAULT_LANG_CTX;
    v_warning NUMBER;
    v_amount PLS_INTEGER;
--    temp varchar2(32767);

    FUNCTION encodeBlob2Base64(pBlobIn IN BLOB) RETURN BLOB IS
      vAmount NUMBER := 45;
      vBlobEnc BLOB := empty_blob();
      vBlobEncLen NUMBER := 0;
      vBlobInLen NUMBER := 0;
      vBuffer RAW(45);
      vOffset NUMBER := 1;
    BEGIN
--      dbms_output.put_line('Start base64 encoding.');
      vBlobInLen := dbms_lob.getlength(pBlobIn);
--      dbms_output.put_line('<BlobInLength>' || vBlobInLen);
      dbms_lob.createtemporary(vBlobEnc, TRUE);
      LOOP
        IF vOffset >= vBlobInLen THEN
          EXIT;
        END IF;
        dbms_lob.read(pBlobIn, vAmount, vOffset, vBuffer);
        BEGIN
          dbms_lob.append(vBlobEnc, utl_encode.base64_encode(vBuffer));
        EXCEPTION
          WHEN OTHERS THEN
          dbms_output.put_line('<vAmount>' || vAmount || '<vOffset>' || vOffset || '<vBuffer>' || vBuffer);
          dbms_output.put_line('ERROR IN append: ' || SQLERRM);
          RAISE;
        END;
        vOffset := vOffset + vAmount;
      END LOOP;
      vBlobEncLen := dbms_lob.getlength(vBlobEnc);
--      dbms_output.put_line('<BlobEncLength>' || vBlobEncLen);
--      dbms_output.put_line('Finshed base64 encoding.');
      RETURN vBlobEnc;
    END encodeBlob2Base64;
  begin
    benc := encodeBlob2Base64(binarydata);
    dbms_lob.createtemporary(c, TRUE);
    v_amount := DBMS_LOB.GETLENGTH(benc);
    DBMS_LOB.CONVERTTOCLOB(c, benc, v_amount, v_clob_offset, v_blob_offset, 1, v_lang_context, v_warning);

    obj := json_value(c);

    dbms_lob.freetemporary(benc);
    dbms_lob.freetemporary(c);
  --dbms_output.put_line(obj.count);
  --dbms_output.put_line(obj.get_last().to_char);
    return obj;

  end encode;

  function decode(v json_value) return blob as
    c clob := empty_clob();
    b blob := empty_blob();
    bret blob;

    v_blob_offset NUMBER := 1;
    v_clob_offset NUMBER := 1;
    v_lang_context NUMBER := 0; --DBMS_LOB.DEFAULT_LANG_CTX;
    v_warning NUMBER;
    v_amount PLS_INTEGER;

    FUNCTION decodeBase642Blob(pBlobIn IN BLOB) RETURN BLOB IS
      vAmount NUMBER := 256;--32;
      vBlobDec BLOB := empty_blob();
      vBlobDecLen NUMBER := 0;
      vBlobInLen NUMBER := 0;
      vBuffer RAW(256);--32);
      vOffset NUMBER := 1;
    BEGIN
--      dbms_output.put_line('Start base64 decoding.');
      vBlobInLen := dbms_lob.getlength(pBlobIn);
--      dbms_output.put_line('<BlobInLength>' || vBlobInLen);
      dbms_lob.createtemporary(vBlobDec, TRUE);
      LOOP
        IF vOffset >= vBlobInLen THEN
          EXIT;
        END IF;
        dbms_lob.read(pBlobIn, vAmount, vOffset, vBuffer);
        BEGIN
          dbms_lob.append(vBlobDec, utl_encode.base64_decode(vBuffer));
        EXCEPTION
          WHEN OTHERS THEN
          dbms_output.put_line('<vAmount>' || vAmount || '<vOffset>' || vOffset || '<vBuffer>' || vBuffer);
          dbms_output.put_line('ERROR IN append: ' || SQLERRM);
          RAISE;
        END;
        vOffset := vOffset + vAmount;
      END LOOP;
      vBlobDecLen := dbms_lob.getlength(vBlobDec);
--      dbms_output.put_line('<BlobDecLength>' || vBlobDecLen);
--      dbms_output.put_line('Finshed base64 decoding.');
      RETURN vBlobDec;
    END decodeBase642Blob;
  begin
    dbms_lob.createtemporary(c, TRUE);
    v.get_string(c);
    v_amount := DBMS_LOB.GETLENGTH(c);
--    dbms_output.put_line('L C'||v_amount);

    dbms_lob.createtemporary(b, TRUE);
    DBMS_LOB.CONVERTTOBLOB(b, c, dbms_lob.lobmaxsize, v_clob_offset, v_blob_offset, 1, v_lang_context, v_warning);
    dbms_lob.freetemporary(c);
    v_amount := DBMS_LOB.GETLENGTH(b);
--    dbms_output.put_line('L B'||v_amount);

    bret := decodeBase642Blob(b);
    dbms_lob.freetemporary(b);
    return bret;

  end decode;


end json_ext;

/
--------------------------------------------------------
--  DDL for Package Body JSON_PARSER
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "JSON_PARSER" as
  /*
  Copyright (c) 2009 Jonas Krogsboell

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  */

  decimalpoint varchar2(1 char) := '.';

  procedure updateDecimalPoint as
  begin
    SELECT substr(VALUE,1,1) into decimalpoint FROM NLS_SESSION_PARAMETERS WHERE PARAMETER = 'NLS_NUMERIC_CHARACTERS';
  end updateDecimalPoint;

  /*type json_src is record (len number, offset number, src varchar2(10), s_clob clob); */
  function next_char(indx number, s in out nocopy json_src) return varchar2 as
  begin
    if(indx > s.len) then return null; end if;
    --right offset?
    if(indx > 4000 + s.offset or indx < s.offset) then
    --load right offset
      s.offset := indx - (indx mod 4000);
      s.src := dbms_lob.substr(s.s_clob, 4000, s.offset+1);
    end if;
    --read from s.src
    return substr(s.src, indx-s.offset, 1);
  end;

  function next_char2(indx number, s in out nocopy json_src, amount number default 1) return varchar2 as
    buf varchar2(32767) := '';
  begin
    for i in 1..amount loop
      buf := buf || next_char(indx-1+i,s);
    end loop;
    return buf;
  end;

  function prepareClob(buf clob) return json_parser.json_src as
    temp json_parser.json_src;
  begin
    temp.s_clob := buf;
    temp.offset := 0;
    temp.src := dbms_lob.substr(buf, 4000, temp.offset+1);
    temp.len := dbms_lob.getlength(buf);
    return temp;
  end;

  function prepareVarchar2(buf varchar2) return json_parser.json_src as
    temp json_parser.json_src;
  begin
    temp.s_clob := buf;
    temp.offset := 0;
    temp.src := substr(buf, 1, 4000);
    temp.len := length(buf);
    return temp;
  end;

  procedure debug(text varchar2) as
  begin
    dbms_output.put_line(text);
  end;

  procedure print_token(t rToken) as
  begin
    dbms_output.put_line('Line: '||t.line||' - Column: '||t.col||' - Type: '||t.type_name||' - Content: '||t.data);
  end print_token;

  /* SCANNER FUNCTIONS START */
  procedure s_error(text varchar2, line number, col number) as
  begin
    raise_application_error(-20100, 'JSON Scanner exception @ line: '||line||' column: '||col||' - '||text);
  end;

  procedure s_error(text varchar2, tok rToken) as
  begin
    raise_application_error(-20100, 'JSON Scanner exception @ line: '||tok.line||' column: '||tok.col||' - '||text);
  end;

  function mt(t varchar2, l pls_integer, c pls_integer, d varchar2) return rToken as
    token rToken;
  begin
    token.type_name := t;
    token.line := l;
    token.col := c;
    token.data := d;
    return token;
  end;

  function lexNumber(jsrc in out nocopy json_src, tok in out nocopy rToken, indx in out nocopy pls_integer) return pls_integer as
    numbuf varchar2(4000) := '';
    buf varchar2(4);
    checkLoop boolean;
  begin
    buf := next_char(indx, jsrc);
    if(buf = '-') then numbuf := '-'; indx := indx + 1; end if;
    buf := next_char(indx, jsrc);
    --0 or [1-9]([0-9])*
    if(buf = '0') then
      numbuf := numbuf || '0'; indx := indx + 1;
      buf := next_char(indx, jsrc);
    elsif(buf >= '1' and buf <= '9') then
      numbuf := numbuf || buf; indx := indx + 1;
      --read digits
      buf := next_char(indx, jsrc);
      while(buf >= '0' and buf <= '9') loop
        numbuf := numbuf || buf; indx := indx + 1;
        buf := next_char(indx, jsrc);
      end loop;
    end if;
    --fraction
    if(buf = '.') then
      numbuf := numbuf || buf; indx := indx + 1;
      buf := next_char(indx, jsrc);
      checkLoop := FALSE;
      while(buf >= '0' and buf <= '9') loop
        checkLoop := TRUE;
        numbuf := numbuf || buf; indx := indx + 1;
        buf := next_char(indx, jsrc);
      end loop;
      if(not checkLoop) then
        s_error('Expected: digits in fraction', tok);
      end if;
    end if;
    --exp part
    if(buf in ('e', 'E')) then
      numbuf := numbuf || buf; indx := indx + 1;
      buf := next_char(indx, jsrc);
      if(buf = '+' or buf = '-') then
        numbuf := numbuf || buf; indx := indx + 1;
        buf := next_char(indx, jsrc);
      end if;
      checkLoop := FALSE;
      while(buf >= '0' and buf <= '9') loop
        checkLoop := TRUE;
        numbuf := numbuf || buf; indx := indx + 1;
        buf := next_char(indx, jsrc);
      end loop;
      if(not checkLoop) then
        s_error('Expected: digits in exp', tok);
      end if;
    end if;

    tok.data := numbuf;
    return indx;
  end lexNumber;

  -- [a-zA-Z]([a-zA-Z0-9])*
  function lexName(jsrc in out nocopy json_src, tok in out nocopy rToken, indx in out nocopy pls_integer) return pls_integer as
    varbuf varchar2(32767) := '';
    buf varchar(4);
    num number;
  begin
    buf := next_char(indx, jsrc);
    while(REGEXP_LIKE(buf, '^[[:alnum:]\_]$', 'i')) loop
      varbuf := varbuf || buf;
      indx := indx + 1;
      buf := next_char(indx, jsrc);
      if (buf is null) then
        goto retname;
        --debug('Premature string ending');
      end if;
    end loop;
    <<retname>>

    --could check for reserved keywords here

    --debug(varbuf);
    tok.data := varbuf;
    return indx-1;
  end lexName;

  procedure updateClob(v_extended in out nocopy clob, v_str varchar2) as
  begin
    dbms_lob.writeappend(v_extended, length(v_str), v_str);
  end updateClob;

  function lexString(jsrc in out nocopy json_src, tok in out nocopy rToken, indx in out nocopy pls_integer, endChar char) return pls_integer as
    v_extended clob := null; v_count number := 0;
    varbuf varchar2(32767) := '';
    buf varchar(4);
    wrong boolean;
  begin
    indx := indx +1;
    buf := next_char(indx, jsrc);
    while(buf != endChar) loop
      --clob control
      if(v_count > 8191) then --crazy oracle error (16383 is the highest working length with unistr - 8192 choosen to be safe)
        if(v_extended is null) then
          v_extended := empty_clob();
          dbms_lob.createtemporary(v_extended, true);
        end if;
        updateClob(v_extended, unistr(varbuf));
        varbuf := ''; v_count := 0;
      end if;
      if(buf = Chr(13) or buf = CHR(9) or buf = CHR(10)) then
        s_error('Control characters not allowed (CHR(9),CHR(10)CHR(13))', tok);
      end if;
      if(buf = '\') then
        --varbuf := varbuf || buf;
        indx := indx + 1;
        buf := next_char(indx, jsrc);
        case
          when buf in ('\') then
            varbuf := varbuf || buf || buf; v_count := v_count + 2;
            indx := indx + 1;
            buf := next_char(indx, jsrc);
          when buf in ('"', '/') then
            varbuf := varbuf || buf; v_count := v_count + 1;
            indx := indx + 1;
            buf := next_char(indx, jsrc);
          when buf = '''' then
            if(json_strict = false) then
              varbuf := varbuf || buf; v_count := v_count + 1;
              indx := indx + 1;
              buf := next_char(indx, jsrc);
            else
              s_error('strictmode - expected: " \ / b f n r t u ', tok);
            end if;
          when buf in ('b', 'f', 'n', 'r', 't') then
            --backspace b = U+0008
            --formfeed  f = U+000C
            --newline   n = U+000A
            --carret    r = U+000D
            --tabulator t = U+0009
            case buf
            when 'b' then varbuf := varbuf || chr(8);
            when 'f' then varbuf := varbuf || chr(13);
            when 'n' then varbuf := varbuf || chr(10);
            when 'r' then varbuf := varbuf || chr(14);
            when 't' then varbuf := varbuf || chr(9);
            end case;
            --varbuf := varbuf || buf;
            v_count := v_count + 1;
            indx := indx + 1;
            buf := next_char(indx, jsrc);
          when buf = 'u' then
            --four hexidecimal chars
            declare
              four varchar2(4);
            begin
              four := next_char2(indx+1, jsrc, 4);
              wrong := FALSE;
              if(upper(substr(four, 1,1)) not in ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','a','b','c','d','e','f')) then wrong := TRUE; end if;
              if(upper(substr(four, 2,1)) not in ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','a','b','c','d','e','f')) then wrong := TRUE; end if;
              if(upper(substr(four, 3,1)) not in ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','a','b','c','d','e','f')) then wrong := TRUE; end if;
              if(upper(substr(four, 4,1)) not in ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','a','b','c','d','e','f')) then wrong := TRUE; end if;
              if(wrong) then
                s_error('expected: " \u([0-9][A-F]){4}', tok);
              end if;
--              varbuf := varbuf || buf || four;
              varbuf := varbuf || '\'||four;--chr(to_number(four,'XXXX'));
               v_count := v_count + 5;
              indx := indx + 5;
              buf := next_char(indx, jsrc);
              end;
          else
            s_error('expected: " \ / b f n r t u ', tok);
        end case;
      else
        varbuf := varbuf || buf; v_count := v_count + 1;
        indx := indx + 1;
        buf := next_char(indx, jsrc);
      end if;
    end loop;

    if (buf is null) then
      s_error('string ending not found', tok);
      --debug('Premature string ending');
    end if;

    --debug(varbuf);
    --dbms_output.put_line(varbuf);
    if(v_extended is not null) then
      updateClob(v_extended, unistr(varbuf));
      tok.data_overflow := v_extended;
      tok.data := dbms_lob.substr(v_extended, 1, 32767);
    else
      tok.data := unistr(varbuf);
    end if;
    return indx;
  end lexString;

  /* scanner tokens:
    '{', '}', ',', ':', '[', ']', STRING, NUMBER, TRUE, FALSE, NULL
  */
  function lexer(jsrc in out nocopy json_src) return lTokens as
    tokens lTokens;
    indx pls_integer := 1;
    tok_indx pls_integer := 1;
    buf varchar2(4);
    lin_no number := 1;
    col_no number := 0;
  begin
    while (indx <= jsrc.len) loop
      --read into buf
      buf := next_char(indx, jsrc);
      col_no := col_no + 1;
      --convert to switch case
      case
        when buf = '{' then tokens(tok_indx) := mt('{', lin_no, col_no, null); tok_indx := tok_indx + 1;
        when buf = '}' then tokens(tok_indx) := mt('}', lin_no, col_no, null); tok_indx := tok_indx + 1;
        when buf = ',' then tokens(tok_indx) := mt(',', lin_no, col_no, null); tok_indx := tok_indx + 1;
        when buf = ':' then tokens(tok_indx) := mt(':', lin_no, col_no, null); tok_indx := tok_indx + 1;
        when buf = '[' then tokens(tok_indx) := mt('[', lin_no, col_no, null); tok_indx := tok_indx + 1;
        when buf = ']' then tokens(tok_indx) := mt(']', lin_no, col_no, null); tok_indx := tok_indx + 1;
        when buf = 't' then
          if(next_char2(indx, jsrc, 4) != 'true') then
            if(json_strict = false and REGEXP_LIKE(buf, '^[[:alpha:]]$', 'i')) then
              tokens(tok_indx) := mt('STRING', lin_no, col_no, null);
              indx := lexName(jsrc, tokens(tok_indx), indx);
              col_no := col_no + length(tokens(tok_indx).data) + 1;
              tok_indx := tok_indx + 1;
            else
              s_error('Expected: ''true''', lin_no, col_no);
            end if;
          else
            tokens(tok_indx) := mt('TRUE', lin_no, col_no, null); tok_indx := tok_indx + 1;
            indx := indx + 3;
            col_no := col_no + 3;
          end if;
        when buf = 'n' then
          if(next_char2(indx, jsrc, 4) != 'null') then
            if(json_strict = false and REGEXP_LIKE(buf, '^[[:alpha:]]$', 'i')) then
              tokens(tok_indx) := mt('STRING', lin_no, col_no, null);
              indx := lexName(jsrc, tokens(tok_indx), indx);
              col_no := col_no + length(tokens(tok_indx).data) + 1;
              tok_indx := tok_indx + 1;
            else
              s_error('Expected: ''null''', lin_no, col_no);
            end if;
          else
            tokens(tok_indx) := mt('NULL', lin_no, col_no, null); tok_indx := tok_indx + 1;
            indx := indx + 3;
            col_no := col_no + 3;
          end if;
        when buf = 'f' then
          if(next_char2(indx, jsrc, 5) != 'false') then
            if(json_strict = false and REGEXP_LIKE(buf, '^[[:alpha:]]$', 'i')) then
              tokens(tok_indx) := mt('STRING', lin_no, col_no, null);
              indx := lexName(jsrc, tokens(tok_indx), indx);
              col_no := col_no + length(tokens(tok_indx).data) + 1;
              tok_indx := tok_indx + 1;
            else
              s_error('Expected: ''false''', lin_no, col_no);
            end if;
          else
            tokens(tok_indx) := mt('FALSE', lin_no, col_no, null); tok_indx := tok_indx + 1;
            indx := indx + 4;
            col_no := col_no + 4;
          end if;
        /*   -- 9 = TAB, 10 = \n, 13 = \r (Linux = \n, Windows = \r\n, Mac = \r */
        when (buf = Chr(10)) then --linux newlines
          lin_no := lin_no + 1;
          col_no := 0;

        when (buf = Chr(13)) then --Windows or Mac way
          lin_no := lin_no + 1;
          col_no := 0;
          if(jsrc.len >= indx +1) then -- better safe than sorry
            buf := next_char(indx+1, jsrc);
            if(buf = Chr(10)) then --\r\n
              indx := indx + 1;
            end if;
          end if;

        when (buf = CHR(9)) then null; --tabbing
        when (buf in ('-', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9')) then --number
          tokens(tok_indx) := mt('NUMBER', lin_no, col_no, null);
          indx := lexNumber(jsrc, tokens(tok_indx), indx)-1;
          col_no := col_no + length(tokens(tok_indx).data);
          tok_indx := tok_indx + 1;
        when buf = '"' then --number
          tokens(tok_indx) := mt('STRING', lin_no, col_no, null);
          indx := lexString(jsrc, tokens(tok_indx), indx, '"');
          col_no := col_no + length(tokens(tok_indx).data) + 1;
          tok_indx := tok_indx + 1;
        when buf = '''' and json_strict = false then --number
          tokens(tok_indx) := mt('STRING', lin_no, col_no, null);
          indx := lexString(jsrc, tokens(tok_indx), indx, '''');
          col_no := col_no + length(tokens(tok_indx).data) + 1; --hovsa her
          tok_indx := tok_indx + 1;
        when json_strict = false and REGEXP_LIKE(buf, '^[[:alpha:]]$', 'i') then
          tokens(tok_indx) := mt('STRING', lin_no, col_no, null);
          indx := lexName(jsrc, tokens(tok_indx), indx);
          if(tokens(tok_indx).data_overflow is not null) then
            col_no := col_no + dbms_lob.getlength(tokens(tok_indx).data_overflow) + 1;
          else
            col_no := col_no + length(tokens(tok_indx).data) + 1;
          end if;
          tok_indx := tok_indx + 1;
        when json_strict = false and buf||next_char(indx+1, jsrc) = '/*' then --strip comments
          declare
            saveindx number := indx;
            un_esc clob;
          begin
            indx := indx + 1;
            loop
              indx := indx + 1;
              buf := next_char(indx, jsrc)||next_char(indx+1, jsrc);
              exit when buf = '*/';
              exit when buf is null;
            end loop;

            if(indx = saveindx+2) then
              --enter unescaped mode
              --dbms_output.put_line('Entering unescaped mode');
              un_esc := empty_clob();
              dbms_lob.createtemporary(un_esc, true);
              indx := indx + 1;
              loop
                indx := indx + 1;
                buf := next_char(indx, jsrc)||next_char(indx+1, jsrc)||next_char(indx+2, jsrc)||next_char(indx+3, jsrc);
                exit when buf = '/**/';
                if buf is null then
                  s_error('Unexpected sequence /**/ to end unescaped data: '||buf, lin_no, col_no);
                end if;
                buf := next_char(indx, jsrc);
                dbms_lob.writeappend(un_esc, length(buf), buf);
              end loop;
              tokens(tok_indx) := mt('ESTRING', lin_no, col_no, null);
              tokens(tok_indx).data_overflow := un_esc;
              col_no := col_no + dbms_lob.getlength(un_esc) + 1; --note: line count won't work properly
              tok_indx := tok_indx + 1;
              indx := indx + 2;
            end if;

            indx := indx + 1;
          end;
        when buf = ' ' then null; --space
        else
          s_error('Unexpected char: '||buf, lin_no, col_no);
      end case;

      indx := indx + 1;
    end loop;

    return tokens;
  end lexer;

  /* SCANNER END */

  /* PARSER FUNCTIONS START*/
  procedure p_error(text varchar2, tok rToken) as
  begin
    raise_application_error(-20101, 'JSON Parser exception @ line: '||tok.line||' column: '||tok.col||' - '||text);
  end;

  function parseObj(tokens lTokens, indx in out nocopy pls_integer) return json;

  function parseArr(tokens lTokens, indx in out nocopy pls_integer) return json_list as
    e_arr json_value_array := json_value_array();
    ret_list json_list := json_list();
    v_count number := 0;
    tok rToken;
  begin
    --value, value, value ]
    if(indx > tokens.count) then p_error('more elements in array was excepted', tok); end if;
    tok := tokens(indx);
    while(tok.type_name != ']') loop
      e_arr.extend;
      v_count := v_count + 1;
      case tok.type_name
        when 'TRUE' then e_arr(v_count) := json_value(true);
        when 'FALSE' then e_arr(v_count) := json_value(false);
        when 'NULL' then e_arr(v_count) := json_value;
        when 'STRING' then e_arr(v_count) := case when tok.data_overflow is not null then json_value(tok.data_overflow) else json_value(tok.data) end;
        when 'ESTRING' then e_arr(v_count) := json_value(tok.data_overflow, false);
        when 'NUMBER' then e_arr(v_count) := json_value(to_number(replace(tok.data, '.', decimalpoint)));
        when '[' then
          declare e_list json_list; begin
            indx := indx + 1;
            e_list := parseArr(tokens, indx);
            e_arr(v_count) := e_list.to_json_value;
          end;
        when '{' then
          indx := indx + 1;
          e_arr(v_count) := parseObj(tokens, indx).to_json_value;
        else
          p_error('Expected a value', tok);
      end case;
      indx := indx + 1;
      if(indx > tokens.count) then p_error('] not found', tok); end if;
      tok := tokens(indx);
      if(tok.type_name = ',') then --advance
        indx := indx + 1;
        if(indx > tokens.count) then p_error('more elements in array was excepted', tok); end if;
        tok := tokens(indx);
        if(tok.type_name = ']') then --premature exit
          p_error('Premature exit in array', tok);
        end if;
      elsif(tok.type_name != ']') then --error
        p_error('Expected , or ]', tok);
      end if;

    end loop;
    ret_list.list_data := e_arr;
    return ret_list;
  end parseArr;

  function parseMem(tokens lTokens, indx in out pls_integer, mem_name varchar2, mem_indx number) return json_value as
    mem json_value;
    tok rToken;
  begin
    tok := tokens(indx);
    case tok.type_name
      when 'TRUE' then mem := json_value(true);
      when 'FALSE' then mem := json_value(false);
      when 'NULL' then mem := json_value;
      when 'STRING' then mem := case when tok.data_overflow is not null then json_value(tok.data_overflow) else json_value(tok.data) end;
      when 'ESTRING' then mem := json_value(tok.data_overflow, false);
      when 'NUMBER' then mem := json_value(to_number(replace(tok.data, '.', decimalpoint)));
      when '[' then
        declare
          e_list json_list;
        begin
          indx := indx + 1;
          e_list := parseArr(tokens, indx);
          mem := e_list.to_json_value;
        end;
      when '{' then
        indx := indx + 1;
        mem := parseObj(tokens, indx).to_json_value;
      else
        p_error('Found '||tok.type_name, tok);
    end case;
    mem.mapname := mem_name;
    mem.mapindx := mem_indx;

    indx := indx + 1;
    return mem;
  end parseMem;

  /*procedure test_duplicate_members(arr in json_member_array, mem_name in varchar2, wheretok rToken) as
  begin
    for i in 1 .. arr.count loop
      if(arr(i).member_name = mem_name) then
        p_error('Duplicate member name', wheretok);
      end if;
    end loop;
  end test_duplicate_members;*/

  function parseObj(tokens lTokens, indx in out nocopy pls_integer) return json as
    type memmap is table of number index by varchar2(4000); -- i've read somewhere that this is not possible - but it is!
    mymap memmap;
    nullelemfound boolean := false;

    obj json;
    tok rToken;
    mem_name varchar(4000);
    arr json_value_array := json_value_array();
  begin
    --what to expect?
    while(indx <= tokens.count) loop
      tok := tokens(indx);
      --debug('E: '||tok.type_name);
      case tok.type_name
      when 'STRING' then
        --member
        mem_name := substr(tok.data, 1, 4000);
        begin
          if(mem_name is null) then
            if(nullelemfound) then
              p_error('Duplicate empty member: ', tok);
            else
              nullelemfound := true;
            end if;
          elsif(mymap(mem_name) is not null) then
            p_error('Duplicate member name: '||mem_name, tok);
          end if;
        exception
          when no_data_found then mymap(mem_name) := 1;
        end;

        indx := indx + 1;
        if(indx > tokens.count) then p_error('Unexpected end of input', tok); end if;
        tok := tokens(indx);
        indx := indx + 1;
        if(indx > tokens.count) then p_error('Unexpected end of input', tok); end if;
        if(tok.type_name = ':') then
          --parse
          declare
            jmb json_value;
            x number;
          begin
            x := arr.count + 1;
            jmb := parseMem(tokens, indx, mem_name, x);
            arr.extend;
            arr(x) := jmb;
          end;
        else
          p_error('Expected '':''', tok);
        end if;
        --move indx forward if ',' is found
        if(indx > tokens.count) then p_error('Unexpected end of input', tok); end if;

        tok := tokens(indx);
        if(tok.type_name = ',') then
          --debug('found ,');
          indx := indx + 1;
          tok := tokens(indx);
          if(tok.type_name = '}') then --premature exit
            p_error('Premature exit in json object', tok);
          end if;
        elsif(tok.type_name != '}') then
           p_error('A comma seperator is probably missing', tok);
        end if;
      when '}' then
        obj := json();
        obj.json_data := arr;
        return obj;
      else
        p_error('Expected string or }', tok);
      end case;
    end loop;

    p_error('} not found', tokens(indx-1));

    return obj;

  end;

  function parser(str varchar2) return json as
    tokens lTokens;
    obj json;
    indx pls_integer := 1;
    jsrc json_src;
  begin
    updateDecimalPoint();
    jsrc := prepareVarchar2(str);
    tokens := lexer(jsrc);
    if(tokens(indx).type_name = '{') then
      indx := indx + 1;
      obj := parseObj(tokens, indx);
    else
      raise_application_error(-20101, 'JSON Parser exception - no { start found');
    end if;
    if(tokens.count != indx) then
      p_error('} should end the JSON object', tokens(indx));
    end if;

    return obj;
  end parser;

  function parse_list(str varchar2) return json_list as
    tokens lTokens;
    obj json_list;
    indx pls_integer := 1;
    jsrc json_src;
  begin
    updateDecimalPoint();
    jsrc := prepareVarchar2(str);
    tokens := lexer(jsrc);
    if(tokens(indx).type_name = '[') then
      indx := indx + 1;
      obj := parseArr(tokens, indx);
    else
      raise_application_error(-20101, 'JSON List Parser exception - no [ start found');
    end if;
    if(tokens.count != indx) then
      p_error('] should end the JSON List object', tokens(indx));
    end if;

    return obj;
  end parse_list;

  function parse_list(str clob) return json_list as
    tokens lTokens;
    obj json_list;
    indx pls_integer := 1;
    jsrc json_src;
  begin
    updateDecimalPoint();
    jsrc := prepareClob(str);
    tokens := lexer(jsrc);
    if(tokens(indx).type_name = '[') then
      indx := indx + 1;
      obj := parseArr(tokens, indx);
    else
      raise_application_error(-20101, 'JSON List Parser exception - no [ start found');
    end if;
    if(tokens.count != indx) then
      p_error('] should end the JSON List object', tokens(indx));
    end if;

    return obj;
  end parse_list;

  function parser(str clob) return json as
    tokens lTokens;
    obj json;
    indx pls_integer := 1;
    jsrc json_src;
  begin
    updateDecimalPoint();
    --dbms_output.put_line('Using clob');
    jsrc := prepareClob(str);
    tokens := lexer(jsrc);
    if(tokens(indx).type_name = '{') then
      indx := indx + 1;
      obj := parseObj(tokens, indx);
    else
      raise_application_error(-20101, 'JSON Parser exception - no { start found');
    end if;
    if(tokens.count != indx) then
      p_error('} should end the JSON object', tokens(indx));
    end if;

    return obj;
  end parser;

  function parse_any(str varchar2) return json_value as
    tokens lTokens;
    obj json_list;
    ret json_value;
    indx pls_integer := 1;
    jsrc json_src;
  begin
    updateDecimalPoint();
    jsrc := prepareVarchar2(str);
    tokens := lexer(jsrc);
    tokens(tokens.count+1).type_name := ']';
    obj := parseArr(tokens, indx);
    if(tokens.count != indx) then
      p_error('] should end the JSON List object', tokens(indx));
    end if;

    return obj.head();
  end parse_any;

  function parse_any(str clob) return json_value as
    tokens lTokens;
    obj json_list;
    indx pls_integer := 1;
    jsrc json_src;
  begin
    jsrc := prepareClob(str);
    tokens := lexer(jsrc);
    tokens(tokens.count+1).type_name := ']';
    obj := parseArr(tokens, indx);
    if(tokens.count != indx) then
      p_error('] should end the JSON List object', tokens(indx));
    end if;

    return obj.head();
  end parse_any;

  /* last entry is the one to keep */
  procedure remove_duplicates(obj in out nocopy json) as
    type memberlist is table of json_value index by varchar2(4000);
    members memberlist;
    nulljsonvalue json_value := null;
    validated json := json();
    indx varchar2(4000);
  begin
    for i in 1 .. obj.count loop
      if(obj.get(i).mapname is null) then
        nulljsonvalue := obj.get(i);
      else
        members(obj.get(i).mapname) := obj.get(i);
      end if;
    end loop;

    validated.check_duplicate(false);
    indx := members.first;
    loop
      exit when indx is null;
      validated.put(indx, members(indx));
      indx := members.next(indx);
    end loop;
    if(nulljsonvalue is not null) then
      validated.put('', nulljsonvalue);
    end if;

    validated.check_for_duplicate := obj.check_for_duplicate;

    obj := validated;
  end;

  function get_version return varchar2 as
  begin
    return 'PL/JSON v1.0.4';
  end get_version;

end json_parser;

/
--------------------------------------------------------
--  DDL for Package Body JSON_PRINTER
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "JSON_PRINTER" as
  max_line_len number := 0;
  cur_line_len number := 0;

  function llcheck(str in varchar2) return varchar2 as
  begin
    --dbms_output.put_line(cur_line_len || ' : '|| str);
    if(max_line_len > 0 and length(str)+cur_line_len > max_line_len) then
      cur_line_len := length(str);
      return newline_char || str;
    else
      cur_line_len := cur_line_len + length(str);
      return str;
    end if;
  end llcheck;

  function escapeString(str varchar2) return varchar2 as
    sb varchar2(32767) := '';
    buf varchar2(40);
    num number;
  begin
    if(str is null) then return ''; end if;
    for i in 1 .. length(str) loop
      buf := substr(str, i, 1);
      --backspace b = U+0008
      --formfeed  f = U+000C
      --newline   n = U+000A
      --carret    r = U+000D
      --tabulator t = U+0009
      case buf
      when chr( 8) then buf := '\b';
      when chr( 9) then buf := '\t';
      when chr(10) then buf := '\n';
      when chr(13) then buf := '\f';
      when chr(14) then buf := '\r';
      when chr(34) then buf := '\"';
      when chr(47) then if(escape_solidus) then buf := '\/'; end if;
      when chr(92) then buf := '\\';
      else
        if(ascii(buf) < 32) then
          buf := '\u'||replace(substr(to_char(ascii(buf), 'XXXX'),2,4), ' ', '0');
        elsif (ascii_output) then
          buf := replace(asciistr(buf), '\', '\u');
        end if;
      end case;

      sb := sb || buf;
    end loop;

    return sb;
  end escapeString;

  function newline(spaces boolean) return varchar2 as
  begin
    cur_line_len := 0;
    if(spaces) then return newline_char; else return ''; end if;
  end;

/*  function get_schema return varchar2 as
  begin
    return sys_context('userenv', 'current_schema');
  end;
*/
  function tab(indent number, spaces boolean) return varchar2 as
    i varchar(200) := '';
  begin
    if(not spaces) then return ''; end if;
    for x in 1 .. indent loop i := i || indent_string; end loop;
    return i;
  end;

  function getCommaSep(spaces boolean) return varchar2 as
  begin
    if(spaces) then return ', '; else return ','; end if;
  end;

  function getMemName(mem json_value, spaces boolean) return varchar2 as
  begin
    if(spaces) then
      return llcheck('"'||escapeString(mem.mapname)||'"') || llcheck(' : ');
    else
      return llcheck('"'||escapeString(mem.mapname)||'"') || llcheck(':');
    end if;
  end;

/* Clob method start here */
  procedure add_to_clob(buf_lob in out nocopy clob, buf_str in out nocopy varchar2, str varchar2) as
  begin
    if(lengthb(str) > 32767 - lengthb(buf_str)) then
--      dbms_lob.append(buf_lob, buf_str);
      dbms_lob.writeappend(buf_lob, length(buf_str), buf_str);
      buf_str := str;
    else
      buf_str := buf_str || str;
    end if;
  end add_to_clob;

  procedure flush_clob(buf_lob in out nocopy clob, buf_str in out nocopy varchar2) as
  begin
--    dbms_lob.append(buf_lob, buf_str);
    dbms_lob.writeappend(buf_lob, length(buf_str), buf_str);
  end flush_clob;

  procedure ppObj(obj json, indent number, buf in out nocopy clob, spaces boolean, buf_str in out nocopy varchar2);

  procedure ppEA(input json_list, indent number, buf in out nocopy clob, spaces boolean, buf_str in out nocopy varchar2) as
    elem json_value;
    arr json_value_array := input.list_data;
    numbuf varchar2(4000);
  begin
    for y in 1 .. arr.count loop
      elem := arr(y);
      if(elem is not null) then
      case elem.get_type
        when 'number' then
          numbuf := '';
          if (elem.get_number < 1 and elem.get_number > 0) then numbuf := '0'; end if;
          if (elem.get_number < 0 and elem.get_number > -1) then
            numbuf := '-0';
            numbuf := numbuf || substr(to_char(elem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,'''),2);
          else
            numbuf := numbuf || to_char(elem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,''');
          end if;
          add_to_clob(buf, buf_str, llcheck(numbuf));
        when 'string' then
          if(elem.extended_str is not null) then --clob implementation
            add_to_clob(buf, buf_str, case when elem.num = 1 then '"' else '/**/' end);
            declare
              offset number := 1;
              v_str varchar(32767);
              amount number := 32767;
            begin
              while(offset <= dbms_lob.getlength(elem.extended_str)) loop
                dbms_lob.read(elem.extended_str, amount, offset, v_str);
                if(elem.num = 1) then
                  add_to_clob(buf, buf_str, escapeString(v_str));
                else
                  add_to_clob(buf, buf_str, v_str);
                end if;
                offset := offset + amount;
              end loop;
            end;
            add_to_clob(buf, buf_str, case when elem.num = 1 then '"' else '/**/' end || newline_char);
          else
            if(elem.num = 1) then
              add_to_clob(buf, buf_str, llcheck('"'||escapeString(elem.get_string)||'"'));
            else
              add_to_clob(buf, buf_str, llcheck('/**/'||elem.get_string||'/**/'));
            end if;
          end if;
        when 'bool' then
          if(elem.get_bool) then
            add_to_clob(buf, buf_str, llcheck('true'));
          else
            add_to_clob(buf, buf_str, llcheck('false'));
          end if;
        when 'null' then
          add_to_clob(buf, buf_str, llcheck('null'));
        when 'array' then
          add_to_clob(buf, buf_str, llcheck('['));
          ppEA(json_list(elem), indent, buf, spaces, buf_str);
          add_to_clob(buf, buf_str, llcheck(']'));
        when 'object' then
          ppObj(json(elem), indent, buf, spaces, buf_str);
        else add_to_clob(buf, buf_str, llcheck(elem.get_type));
      end case;
      end if;
      if(y != arr.count) then add_to_clob(buf, buf_str, llcheck(getCommaSep(spaces))); end if;
    end loop;
  end ppEA;

  procedure ppMem(mem json_value, indent number, buf in out nocopy clob, spaces boolean, buf_str in out nocopy varchar2) as
    numbuf varchar2(4000);
  begin
    add_to_clob(buf, buf_str, llcheck(tab(indent, spaces)) || llcheck(getMemName(mem, spaces)));
    case mem.get_type
      when 'number' then
        if (mem.get_number < 1 and mem.get_number > 0) then numbuf := '0'; end if;
        if (mem.get_number < 0 and mem.get_number > -1) then
          numbuf := '-0';
          numbuf := numbuf || substr(to_char(mem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,'''),2);
        else
          numbuf := numbuf || to_char(mem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,''');
        end if;
        add_to_clob(buf, buf_str, llcheck(numbuf));
      when 'string' then
        if(mem.extended_str is not null) then --clob implementation
          add_to_clob(buf, buf_str, case when mem.num = 1 then '"' else '/**/' end);
          declare
            offset number := 1;
            v_str varchar(32767);
            amount number := 32767;
          begin
--            dbms_output.put_line('SIZE:'||dbms_lob.getlength(mem.extended_str));
            while(offset <= dbms_lob.getlength(mem.extended_str)) loop
--            dbms_output.put_line('OFFSET:'||offset);
 --             v_str := dbms_lob.substr(mem.extended_str, 8192, offset);
              dbms_lob.read(mem.extended_str, amount, offset, v_str);
--            dbms_output.put_line('VSTR_SIZE:'||length(v_str));
              if(mem.num = 1) then
                add_to_clob(buf, buf_str, escapeString(v_str));
              else
                add_to_clob(buf, buf_str, v_str);
              end if;
              offset := offset + amount;
            end loop;
          end;
          add_to_clob(buf, buf_str, case when mem.num = 1 then '"' else '/**/' end || newline_char);
        else
          if(mem.num = 1) then
            add_to_clob(buf, buf_str, llcheck('"'||escapeString(mem.get_string)||'"'));
          else
            add_to_clob(buf, buf_str, llcheck('/**/'||mem.get_string||'/**/'));
          end if;
        end if;
      when 'bool' then
        if(mem.get_bool) then
          add_to_clob(buf, buf_str, llcheck('true'));
        else
          add_to_clob(buf, buf_str, llcheck('false'));
        end if;
      when 'null' then
        add_to_clob(buf, buf_str, llcheck('null'));
      when 'array' then
        add_to_clob(buf, buf_str, llcheck('['));
        ppEA(json_list(mem), indent, buf, spaces, buf_str);
        add_to_clob(buf, buf_str, llcheck(']'));
      when 'object' then
        ppObj(json(mem), indent, buf, spaces, buf_str);
      else add_to_clob(buf, buf_str, llcheck(mem.get_type));
    end case;
  end ppMem;

  procedure ppObj(obj json, indent number, buf in out nocopy clob, spaces boolean, buf_str in out nocopy varchar2) as
  begin
    add_to_clob(buf, buf_str, llcheck('{') || newline(spaces));
    for m in 1 .. obj.json_data.count loop
      ppMem(obj.json_data(m), indent+1, buf, spaces, buf_str);
      if(m != obj.json_data.count) then
        add_to_clob(buf, buf_str, llcheck(',') || newline(spaces));
      else
        add_to_clob(buf, buf_str, newline(spaces));
      end if;
    end loop;
    add_to_clob(buf, buf_str, llcheck(tab(indent, spaces)) || llcheck('}')); -- || chr(13);
  end ppObj;

  procedure pretty_print(obj json, spaces boolean default true, buf in out nocopy clob, line_length number default 0, erase_clob boolean default true) as
    buf_str varchar2(32767);
    amount number := dbms_lob.getlength(buf);
  begin
    if(erase_clob and amount > 0) then dbms_lob.trim(buf, 0); dbms_lob.erase(buf, amount); end if;

    max_line_len := line_length;
    cur_line_len := 0;
    ppObj(obj, 0, buf, spaces, buf_str);
    flush_clob(buf, buf_str);
  end;

  procedure pretty_print_list(obj json_list, spaces boolean default true, buf in out nocopy clob, line_length number default 0, erase_clob boolean default true) as
    buf_str varchar2(32767);
    amount number := dbms_lob.getlength(buf);
  begin
    if(erase_clob and amount > 0) then dbms_lob.trim(buf, 0); dbms_lob.erase(buf, amount); end if;

    max_line_len := line_length;
    cur_line_len := 0;
    add_to_clob(buf, buf_str, llcheck('['));
    ppEA(obj, 0, buf, spaces, buf_str);
    add_to_clob(buf, buf_str, llcheck(']'));
    flush_clob(buf, buf_str);
  end;

  procedure pretty_print_any(json_part json_value, spaces boolean default true, buf in out nocopy clob, line_length number default 0, erase_clob boolean default true) as
    buf_str varchar2(32767) := '';
    numbuf varchar2(4000);
    amount number := dbms_lob.getlength(buf);
  begin
    if(erase_clob and amount > 0) then dbms_lob.trim(buf, 0); dbms_lob.erase(buf, amount); end if;

    case json_part.get_type
      when 'number' then
        if (json_part.get_number < 1 and json_part.get_number > 0) then numbuf := '0'; end if;
        if (json_part.get_number < 0 and json_part.get_number > -1) then
          numbuf := '-0';
          numbuf := numbuf || substr(to_char(json_part.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,'''),2);
        else
          numbuf := numbuf || to_char(json_part.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,''');
        end if;
        add_to_clob(buf, buf_str, numbuf);
      when 'string' then
        if(json_part.extended_str is not null) then --clob implementation
          add_to_clob(buf, buf_str, case when json_part.num = 1 then '"' else '/**/' end);
          declare
            offset number := 1;
            v_str varchar(32767);
            amount number := 32767;
          begin
            while(offset <= dbms_lob.getlength(json_part.extended_str)) loop
              dbms_lob.read(json_part.extended_str, amount, offset, v_str);
              if(json_part.num = 1) then
                add_to_clob(buf, buf_str, escapeString(v_str));
              else
                add_to_clob(buf, buf_str, v_str);
              end if;
              offset := offset + amount;
            end loop;
          end;
          add_to_clob(buf, buf_str, case when json_part.num = 1 then '"' else '/**/' end);
        else
          if(json_part.num = 1) then
            add_to_clob(buf, buf_str, llcheck('"'||escapeString(json_part.get_string)||'"'));
          else
            add_to_clob(buf, buf_str, llcheck('/**/'||json_part.get_string||'/**/'));
          end if;
        end if;
      when 'bool' then
	      if(json_part.get_bool) then
          add_to_clob(buf, buf_str, 'true');
        else
          add_to_clob(buf, buf_str, 'false');
        end if;
      when 'null' then
        add_to_clob(buf, buf_str, 'null');
      when 'array' then
        pretty_print_list(json_list(json_part), spaces, buf, line_length);
        return;
      when 'object' then
        pretty_print(json(json_part), spaces, buf, line_length);
        return;
      else add_to_clob(buf, buf_str, 'unknown type:'|| json_part.get_type);
    end case;
    flush_clob(buf, buf_str);
  end;

/* Clob method end here */

/* Varchar2 method start here */

  procedure ppObj(obj json, indent number, buf in out nocopy varchar2, spaces boolean);

  procedure ppEA(input json_list, indent number, buf in out varchar2, spaces boolean) as
    elem json_value;
    arr json_value_array := input.list_data;
    str varchar2(400);
  begin
    for y in 1 .. arr.count loop
      elem := arr(y);
      if(elem is not null) then
      case elem.get_type
        when 'number' then
          str := '';
          if (elem.get_number < 1 and elem.get_number > 0) then str := '0'; end if;
          if (elem.get_number < 0 and elem.get_number > -1) then
            str := '-0' || substr(to_char(elem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,'''),2);
          else
            str := str || to_char(elem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,''');
          end if;
          buf := buf || llcheck(str);
        when 'string' then
          if(elem.num = 1) then
            buf := buf || llcheck('"'||escapeString(elem.get_string)||'"');
          else
            buf := buf || llcheck('/**/'||elem.get_string||'/**/');
          end if;
        when 'bool' then
          if(elem.get_bool) then
            buf := buf || llcheck('true');
          else
            buf := buf || llcheck('false');
          end if;
        when 'null' then
          buf := buf || llcheck('null');
        when 'array' then
          buf := buf || llcheck('[');
          ppEA(json_list(elem), indent, buf, spaces);
          buf := buf || llcheck(']');
        when 'object' then
          ppObj(json(elem), indent, buf, spaces);
        else buf := buf || llcheck(elem.get_type); /* should never happen */
      end case;
      end if;
      if(y != arr.count) then buf := buf || llcheck(getCommaSep(spaces)); end if;
    end loop;
  end ppEA;

  procedure ppMem(mem json_value, indent number, buf in out nocopy varchar2, spaces boolean) as
    str varchar2(400) := '';
  begin
    buf := buf || llcheck(tab(indent, spaces)) || getMemName(mem, spaces);
    case mem.get_type
      when 'number' then
        if (mem.get_number < 1 and mem.get_number > 0) then str := '0'; end if;
        if (mem.get_number < 0 and mem.get_number > -1) then
          str := '-0' || substr(to_char(mem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,'''),2);
        else
          str := str || to_char(mem.get_number, 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,''');
        end if;
        buf := buf || llcheck(str);
      when 'string' then
        if(mem.num = 1) then
          buf := buf || llcheck('"'||escapeString(mem.get_string)||'"');
        else
          buf := buf || llcheck('/**/'||mem.get_string||'/**/');
        end if;
      when 'bool' then
        if(mem.get_bool) then
          buf := buf || llcheck('true');
        else
          buf := buf || llcheck('false');
        end if;
      when 'null' then
        buf := buf || llcheck('null');
      when 'array' then
        buf := buf || llcheck('[');
        ppEA(json_list(mem), indent, buf, spaces);
        buf := buf || llcheck(']');
      when 'object' then
        ppObj(json(mem), indent, buf, spaces);
      else buf := buf || llcheck(mem.get_type); /* should never happen */
    end case;
  end ppMem;

  procedure ppObj(obj json, indent number, buf in out nocopy varchar2, spaces boolean) as
  begin
    buf := buf || llcheck('{') || newline(spaces);
    for m in 1 .. obj.json_data.count loop
      ppMem(obj.json_data(m), indent+1, buf, spaces);
      if(m != obj.json_data.count) then buf := buf || llcheck(',') || newline(spaces);
      else buf := buf || newline(spaces); end if;
    end loop;
    buf := buf || llcheck(tab(indent, spaces)) || llcheck('}'); -- || chr(13);
  end ppObj;

  function pretty_print(obj json, spaces boolean default true, line_length number default 0) return varchar2 as
    buf varchar2(32767) := '';
  begin
    max_line_len := line_length;
    cur_line_len := 0;
    ppObj(obj, 0, buf, spaces);
    return buf;
  end pretty_print;

  function pretty_print_list(obj json_list, spaces boolean default true, line_length number default 0) return varchar2 as
    buf varchar2(32767);
  begin
    max_line_len := line_length;
    cur_line_len := 0;
    buf := llcheck('[');
    ppEA(obj, 0, buf, spaces);
    buf := buf || llcheck(']');
    return buf;
  end;

  function pretty_print_any(json_part json_value, spaces boolean default true, line_length number default 0) return varchar2 as
    buf varchar2(32767) := '';
  begin
    case json_part.get_type
      when 'number' then
        if (json_part.get_number() < 1 and json_part.get_number() > 0) then buf := buf || '0'; end if;
        if (json_part.get_number() < 0 and json_part.get_number() > -1) then
          buf := buf || '-0';
          buf := buf || substr(to_char(json_part.get_number(), 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,'''),2);
        else
          buf := buf || to_char(json_part.get_number(), 'TM9', 'NLS_NUMERIC_CHARACTERS=''.,''');
        end if;
      when 'string' then
        if(json_part.num = 1) then
          buf := buf || '"'||escapeString(json_part.get_string)||'"';
        else
          buf := buf || '/**/'||json_part.get_string||'/**/';
        end if;
      when 'bool' then
      	if(json_part.get_bool) then buf := 'true'; else buf := 'false'; end if;
      when 'null' then
        buf := 'null';
      when 'array' then
        buf := pretty_print_list(json_list(json_part), spaces, line_length);
      when 'object' then
        buf := pretty_print(json(json_part), spaces, line_length);
      else buf := 'weird error: '|| json_part.get_type;
    end case;
    return buf;
  end;

  procedure dbms_output_clob(my_clob clob, delim varchar2, jsonp varchar2 default null) as
    prev number := 1;
    indx number := 1;
    size_of_nl number := lengthb(delim);
    v_str varchar2(32767);
    amount number := 32767;
  begin
    if(jsonp is not null) then dbms_output.put_line(jsonp||'('); end if;
    while(indx != 0) loop
      --read every line
      indx := dbms_lob.instr(my_clob, delim, prev+1);
 --     dbms_output.put_line(prev || ' to ' || indx);

      if(indx = 0) then
        --emit from prev to end;
        amount := 32767;
 --       dbms_output.put_line(' mycloblen ' || dbms_lob.getlength(my_clob));
        loop
          dbms_lob.read(my_clob, amount, prev, v_str);
          dbms_output.put_line(v_str);
          prev := prev+amount-1;
          exit when prev >= dbms_lob.getlength(my_clob);
        end loop;
      else
        amount := indx - prev;
        if(amount > 32767) then
          amount := 32767;
--          dbms_output.put_line(' mycloblen ' || dbms_lob.getlength(my_clob));
          loop
            dbms_lob.read(my_clob, amount, prev, v_str);
            dbms_output.put_line(v_str);
            prev := prev+amount-1;
            amount := indx - prev;
            exit when prev >= indx - 1;
            if(amount > 32767) then amount := 32767; end if;
          end loop;
          prev := indx + size_of_nl;
        else
          dbms_lob.read(my_clob, amount, prev, v_str);
          dbms_output.put_line(v_str);
          prev := indx + size_of_nl;
        end if;
      end if;

    end loop;
    if(jsonp is not null) then dbms_output.put_line(')'); end if;

/*    while (amount != 0) loop
      indx := dbms_lob.instr(my_clob, delim, prev+1);

--      dbms_output.put_line(prev || ' to ' || indx);
      if(indx = 0) then
        indx := dbms_lob.getlength(my_clob)+1;
      end if;

      if(indx-prev > 32767) then
        indx := prev+32767;
      end if;
--      dbms_output.put_line(prev || ' to ' || indx);
      --substr doesnt work properly on all platforms! (come on oracle - error on Oracle VM for virtualbox)
--        dbms_output.put_line(dbms_lob.substr(my_clob, indx-prev, prev));
      amount := indx-prev;
--        dbms_output.put_line('amount'||amount);
      dbms_lob.read(my_clob, amount, prev, v_str);
      dbms_output.put_line(v_str);
      prev := indx+size_of_nl;
      if(amount = 32767) then prev := prev-size_of_nl-1; end if;
    end loop;
    if(jsonp is not null) then dbms_output.put_line(')'); end if;*/
  end;


/*  procedure dbms_output_clob(my_clob clob, delim varchar2, jsonp varchar2 default null) as
    prev number := 1;
    indx number := 1;
    size_of_nl number := lengthb(delim);
    v_str varchar2(32767);
    amount number;
  begin
    if(jsonp is not null) then dbms_output.put_line(jsonp||'('); end if;
    while (indx != 0) loop
      indx := dbms_lob.instr(my_clob, delim, prev+1);

--      dbms_output.put_line(prev || ' to ' || indx);
      if(indx-prev > 32767) then
        indx := prev+32767;
      end if;
--      dbms_output.put_line(prev || ' to ' || indx);
      --substr doesnt work properly on all platforms! (come on oracle - error on Oracle VM for virtualbox)
      if(indx = 0) then
--        dbms_output.put_line(dbms_lob.substr(my_clob, dbms_lob.getlength(my_clob)-prev+size_of_nl, prev));
        amount := dbms_lob.getlength(my_clob)-prev+size_of_nl;
        dbms_lob.read(my_clob, amount, prev, v_str);
      else
--        dbms_output.put_line(dbms_lob.substr(my_clob, indx-prev, prev));
        amount := indx-prev;
--        dbms_output.put_line('amount'||amount);
        dbms_lob.read(my_clob, amount, prev, v_str);
      end if;
      dbms_output.put_line(v_str);
      prev := indx+size_of_nl;
      if(amount = 32767) then prev := prev-size_of_nl-1; end if;
    end loop;
    if(jsonp is not null) then dbms_output.put_line(')'); end if;
  end;
*/
  procedure htp_output_clob(my_clob clob, jsonp varchar2 default null) as
    /*amount number := 4096;
    pos number := 1;
    len number;
    */
    l_amt    number default 30;
    l_off   number default 1;
    l_str   varchar2(4096);

  begin
    if(jsonp is not null) then htp.prn(jsonp||'('); end if;

    begin
      loop
        dbms_lob.read( my_clob, l_amt, l_off, l_str );

        -- it is vital to use htp.PRN to avoid
        -- spurious line feeds getting added to your
        -- document
        htp.prn( l_str  );
        l_off := l_off+l_amt;
        l_amt := 4096;
      end loop;
    exception
      when no_data_found then NULL;
    end;

    /*
    len := dbms_lob.getlength(my_clob);

    while(pos < len) loop
      htp.prn(dbms_lob.substr(my_clob, amount, pos)); -- should I replace substr with dbms_lob.read?
      --dbms_output.put_line(dbms_lob.substr(my_clob, amount, pos));
      pos := pos + amount;
    end loop;
    */
    if(jsonp is not null) then htp.prn(')'); end if;
  end;

end json_printer;

/
--------------------------------------------------------
--  DDL for Package Body RDM_AUTH
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_AUTH" is

function login(
  p_username in varchar2,
  p_password in varchar2) return boolean is
begin
  plog.info('Prisijungė naudotojas '||p_username);
  return true;
end;

procedure logout is
begin
  plog.info('Atsijungimas nuo sistemos');
  owa_util.redirect_url(rdm_prm.get_value('LOGOUT_URL'), true);
end;

function check_min(
  p_eppn in varchar2,
  p_affiliation in varchar2) return number is

  cursor c_min(
    p_min_vardas in varchar2,
    p_min_tipas in varchar2) is
  select
    min_id
  from
    rdm_nariai
  where
    min_vardas = p_min_vardas and
    min_tipas = p_min_tipas;

  l_min_id number;
  l_min_tipas varchar2(2);
begin
  l_min_tipas := get_min_type(p_affiliation);
  open c_min(p_eppn, l_min_tipas);
  fetch c_min into l_min_id;
  close c_min;
  if l_min_id is null then
    l_min_id := ins(p_eppn, l_min_tipas);
  end if;

  return l_min_id;
end;

/*
  Naujas įrašas rdm_naudotojai
*/
function ins(
  p_min_vardas in varchar2,
  p_min_tipas in varchar2) return number is

  cursor c_lyg is
  select
    lyg_id
  from
    rdm_lygmenys;

  l_min_id number;
  l_lyg_id number;
begin
  /*Priskiriame naujam naudotojui bet kok? lygmen?. V?liau ?vykdome proced?r? check_istaiga*/
  open c_lyg;
  fetch c_lyg into l_lyg_id;
  close c_lyg;

  insert into rdm_nariai(
    min_id,
    min_vardas,
    min_tipas,
    min_lyg_id)
  values(
    rdm_min_id#.nextval,
    p_min_vardas,
    p_min_tipas,
    l_lyg_id)
  returning
    min_id
  into
    l_min_id;

  return l_min_id;
end;

function get_min_type(
  p_affiliation in varchar2) return varchar2 is

  l_result varchar2(2);
begin
  if instr(upper(p_affiliation), 'EMPLOYEE') > 0 then
    l_result := 'D';
  elsif instr(upper(p_affiliation), 'STUDENT') > 0 then
    l_result := 'S';
  else
    l_result := '';
  end if;

  return l_result;
end;

procedure check_istaiga(
  p_org_kodas in varchar2,
  p_min_id in number) is

  cursor c_org(
    p_org_kodas in varchar2) is
  select
    org_id
  from
    rdm_organizacijos
  where
    upper(org_kodas) = upper(p_org_kodas);

  cursor c_lyg(
    p_min_id in number,
    p_org_id in number) is
  select
    lyg_id
  from
    rdm_nariai min inner join
    rdm_lygmenys lyg on lyg.lyg_id = min.min_lyg_id inner join
    rdm_organizacijos org on org.org_id = lyg.lyg_org_id
  where
    min_id = p_min_id and
    org_id = p_org_id;

  l_org_id number;
  l_lyg_id number;
begin
  open c_org(p_org_kodas);
  fetch c_org into l_org_id;
  close c_org;

  open c_lyg(p_min_id, l_org_id);
  fetch c_lyg into l_lyg_id;
  close c_lyg;
  if l_lyg_id is null then
    update rdm_nariai set
      min_lyg_id = (select
                     lyg_id
                   from
                     rdm_lygmenys lyg inner join
                     rdm_organizacijos org on org_id = lyg_org_id
                   where
                     org_id = l_org_id and rownum = 1)
    where
      min_id = p_min_id;
  end if;
end;

procedure check_istaiga_existance(
  p_org_kodas in varchar2) is

  cursor c_org(
    p_org_kodas in varchar2) is
  select
    org_id
  from
    rdm_organizacijos
  where
    upper(org_kodas) = upper(p_org_kodas);

  l_org_id number;
  l_lyg_id number;
begin
  open c_org(p_org_kodas);
  fetch c_org into l_org_id;
  close c_org;
  if l_org_id is null then
    /*nauja organizacija*/
    insert into rdm_organizacijos(
      org_id,
      org_pavadinimas,
      org_kodas)
    values(
      rdm_org_id#.nextval,
      p_org_kodas,
      p_org_kodas)
    returning
      org_id
    into
      l_org_id;

    /*naujas lygmuo*/
    l_lyg_id := rdm_lyg_id#.nextval;
    insert into rdm_lygmenys(
      lyg_id,
      lyg_pavadinimas,
      lyg_org_id,
      lyg_kodas)
    values(
      l_lyg_id,
      p_org_kodas,
      l_org_id,
      p_org_kodas);
  end if;
end;

function get_roles(
  p_elaba_roles in varchar2,
  p_affiliation in varchar2) return t_roles is

  l_result t_roles;
begin
  /* eLABa rolės sumappinamos su mainavietės rolėmis */
  if instr(p_elaba_roles, C_DOC_DEP_EDIT_EPAS) > 0 then
    /* Padalinio administratorius */
    l_result(C_DEPARTMENT_ADM) := 'T';
    plog.debug('Padalinio administratorius');
  end if;
  if instr(p_elaba_roles, C_DOC_INS_EDIT_EPAS) > 0 then
    /* Institucijos administratorius */
    l_result(C_INSTITUTION_ADM) := 'T';
    plog.debug('Institucijos administratorius');
  end if;
  /* edupersonaffiliation sumapinamas su mainavietės rolėmis */
  if instr(upper(p_affiliation), 'EMPLOYEE') > 0 then
    /* Dėstytojas */
    l_result(C_TUTOR) := 'T';
    plog.debug('Dėstytojas');
  elsif instr(upper(p_affiliation), 'STUDENT') > 0 then
    /* Studentas */
    l_result(C_STUDENT) := 'T';
    plog.debug('Studentas');
  end if;

  return l_result;
end;

/*
  Funkcija transformuoja Shibboleth atributus sistemos autorizavimo poreikiams:
  1. Užpildo naudotojo id.
  2. Naudotojo vardas.
  3. El. paštas.
  4. Užpildo instituciją, su kuria susijęs naudotojas.
  5. Užpildo institucijos padalinius, su kuriais susijęs naudotojas.
  6. Užpildo naudotojo turimas roles.
  7. Užpildo naudotojo vardą.
  8. Užpildo naudotojo pavardę.
  9. Užpildo naudotojo tipą (student, employee).
*/
function get_context(
  p_shibboleth_attributes shibboleth.t_attributes) return t_context is

  l_result t_context;
begin
  l_result.naudotojo_id := p_shibboleth_attributes.edupersontargetedid;
  l_result.naudotojo_vardas := p_shibboleth_attributes.displayname;
  l_result.el_pastas := p_shibboleth_attributes.email;
  l_result.institucijos_id := p_shibboleth_attributes.elabainstitutionid;
  l_result.institucijos_pavad := p_shibboleth_attributes.schachomeorganization;
  /* eLABa SSO padalinius perduoda formatu id1;id2;id3. Transformuojame į atskirus įrašus */
  l_result.padaliniai := rdm_utl.split(p_shibboleth_attributes.elabadutydepartments, rdm_utl.C_SEMICOLON);

  /*
    Užpildo naudotojo turimas roles. Rolės saugomos atribute elabaprivileges. SSO perduoda
    visas eLABa turimas roles, todėl išsirenkame tik tas, kurios aktualios. Taip pat rolės
    nustatomos iš atributo edupersonaffiliation.
  */
  l_result.roles := get_roles(p_shibboleth_attributes.elabaprivileges, p_shibboleth_attributes.edupersonaffiliation);

  l_result.vardas := p_shibboleth_attributes.givenname;
  l_result.pavarde := p_shibboleth_attributes.surname;
  l_result.tipas := p_shibboleth_attributes.edupersonaffiliation;

  plog.debug('Naudotojo Id:'||l_result.naudotojo_id);
  plog.debug('Naudotojo vardas:'||l_result.naudotojo_vardas);
  plog.debug('Naudotojo el. paštas:'||l_result.el_pastas);
  plog.debug('Naudotojo institucijos Id:'||l_result.institucijos_id);
  plog.debug('Naudotojo institucijos pavadinimas:'||l_result.institucijos_pavad);

  return l_result;
end;

/*
  UNIT tests
*/
procedure test_get_tutor_context is

  l_shibboleth_attributes shibboleth.t_attributes;
  l_result t_context;
  l_index pls_integer;
  l_index_chr varchar2(20);
begin
  l_shibboleth_attributes := shibboleth.test_get_tutor_attributes;
  l_result := get_context(l_shibboleth_attributes);
  dbms_output.put_line(l_result.naudotojo_id);
  dbms_output.put_line(l_result.naudotojo_vardas);
  dbms_output.put_line(l_result.el_pastas);
  dbms_output.put_line(l_result.institucijos_id);
  dbms_output.put_line(l_result.institucijos_pavad);
  dbms_output.put_line('Padaliniai');
  l_index := l_result.padaliniai.first;
  while l_index is not null loop
    dbms_output.put_line(l_result.padaliniai(l_index));
    l_index := l_result.padaliniai.next(l_index);
  end loop;
  dbms_output.put_line('Roles');
  l_index_chr := l_result.roles.first;
  while l_index_chr is not null loop
    --dbms_output.put_line(l_result.roles(l_index_chr));
    dbms_output.put_line(l_index_chr);
    l_index_chr := l_result.roles.next(l_index_chr);
  end loop;
end;

function test_set_tutor_context return t_context is

begin
  return get_context(shibboleth.test_get_tutor_attributes);
end;

function test_set_stud_context return t_context is

begin
  return get_context(shibboleth.test_get_stud_attributes);
end;

end rdm_auth;

/
--------------------------------------------------------
--  DDL for Package Body RDM_JOB_PROCEDURES
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_JOB_PROCEDURES" is

/*
  Procedūra išsiunčia dar nesiųstus dokumentus į EPAS.
  Iš EPAs grįžta atsakymas: ataskaitos id
*/
procedure send_docs_to_epas is
  cursor c_darbai is
  select
    rdb_id
  from
    rdm_darbai
  where
    rdb_statusas = 'I';
begin
  for cur in c_darbai
  loop
    rdm_ws.get_oa_id(cur.rdb_id);
  end loop;
  commit;
end;

/*
  Procedūra atsisiunčia iš EPAS dokumento panašumo procentą
*/
procedure set_similarity_score is
  cursor c_darbai is
  select
    rdb_id,
    rdb_epas_oa_id
  from
    rdm_darbai
  where
    rdb_epas_oa_id is not null and
    rdb_nebetikrinti_procento = 0;

  l_sim_score_ws rdm_ws.t_similarity_score_ws;
begin
  for c_darbas in c_darbai loop
    l_sim_score_ws := rdm_ws.get_sim_score(c_darbas.rdb_epas_oa_id);
    if l_sim_score_ws.similarity_score is not null or l_sim_score_ws.report_status in (5,6,10) then
      rdm_rdb.set_sim_score(c_darbas.rdb_id, l_sim_score_ws.similarity_score, 0);
    end if;
  end loop;
  commit;
end;

/*
  Įjungti foninį darbą
*/
procedure enable_job(
  p_job_name in varchar2) is
begin
  dbms_scheduler.enable(p_job_name);
  plog.info('Įjungtas foninis darbas '||p_job_name);
end;

/*
  Išjungti foninį darbą
*/
procedure disable_job(
  p_job_name in varchar2) is
begin
  dbms_scheduler.disable(p_job_name);
  plog.info('Išungtas foninis darbas '||p_job_name);
end;

/*
  Pakeisti foninio darbo periodiškumą
*/
procedure alter_job(
  p_name in varchar2,
  p_value in number,
  p_freq in varchar2) is
begin
  dbms_scheduler.set_attribute(
    name => p_name,
    attribute => 'repeat_interval',
    value => 'FREQ='|| p_freq ||';INTERVAL=' || round(p_value));
  plog.info('Pakeistas foninio darbo '||p_name||' periodiškumas: FREQ='|| p_freq ||';INTERVAL=' || round(p_value));
end;

end RDM_JOB_PROCEDURES;

/
--------------------------------------------------------
--  DDL for Package Body RDM_LYG
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_LYG" is
/*
  Konstantos
*/
PACKAGE_NAME constant varchar2(30) := 'rdm_lyg';

/*
  I?ankstiniai paprogrami¸ prototipai
*/

procedure get(
  p_id in number,
  p_val in out nocopy t_val,
  p_raise_error in boolean default true);

procedure maintain(
  p_ins in boolean,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val);

procedure post_process(
  p_operation in varchar2,
  p_ind in t_ind,
  p_new in t_val,
  p_old in t_val);

procedure pre_process(
  p_operation in varchar2,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val);

/*
  Paprogram?s
*/

procedure check_delete(
  p_id in number) is
begin
  declare
    cursor c_rdb(
      p_lyg_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_darbai
    where
      rdb_lyg_id = p_lyg_id;
    l_dummy varchar2(1);
  begin
    open c_rdb(p_id);
    fetch c_rdb into l_dummy;
    close c_rdb;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_LYG_FK');
    end if;
  end;
  declare
    cursor c_min(
      p_lyg_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_nariai
    where
      min_lyg_id = p_lyg_id;
    l_dummy varchar2(1);
  begin
    open c_min(p_id);
    fetch c_min into l_dummy;
    close c_min;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDM_MIN_LYG_FK');
    end if;
  end;
end;

procedure del(
  p_id in number,
  p_raise_error in boolean default true) is
  l_ind t_ind;
  l_new t_val;
  l_old t_val;
begin
  get(p_id, l_old, p_raise_error);
  if l_old.lyg_id is not null then
    pre_process('DEL', l_ind, l_new, l_old);
    delete /*+first_rows(1)*/ rdm_lygmenys
    where
      lyg_id = p_id;
    if sql%rowcount != 0 then
      post_process('DEL', l_ind, l_new, l_old);
    else
      if p_raise_error then
        raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.del');
      end if;
    end if;
  end if;
end;

function find_uk(
  p_kodas in varchar2,
  p_org_id in number,
  p_raise_error in boolean default true) return rdm_lyg.t_val is

  cursor c_tab(
    p_kodas in varchar2,
    p_org_id in number) is
  select /*+first_rows(1)*/
    *
  from
    rdm_lygmenys
  where
    lyg_kodas = p_kodas and
    lyg_org_id = p_org_id;
  l_result rdm_lyg.t_val;
begin
  open c_tab(
    p_kodas,
    p_org_id);
  fetch c_tab into l_result;
  close c_tab;
  if l_result.lyg_id is null and p_raise_error then
    raise_application_error(-20001, 'RDM_LYG_UK');
  end if;
  return l_result;
end;

procedure get(
  p_id in number,
  p_val in out nocopy t_val,
  p_raise_error in boolean default true) is
  cursor c(
    p_id in number) is
  select /*+first_rows(1)*/
    lyg_id,
    lyg_pavadinimas,
    lyg_org_id,
    lyg_kodas,
    lyg_adresas
  from
    rdm_lygmenys
  where
    lyg_id = p_id;
begin
  open c(p_id);
  fetch c into p_val;
  close c;
  if p_val.lyg_id is null and p_raise_error then
    raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.get');
  end if;
end;

function get_lyg_id return number is
  cursor c is
  select
    rdm_lyg_id#.nextval
  from
    dual;
  l_result number;
begin
  open c;
  fetch c into l_result;
  close c;
  return l_result;
end;

procedure ins(
  p_rec in out nocopy t_rec) is
begin
  pre_process('INS', p_rec.i, p_rec.v, null);
  insert into rdm_lygmenys(
    lyg_id,
    lyg_pavadinimas,
    lyg_org_id,
    lyg_kodas,
    lyg_adresas)
  values(
    p_rec.v.lyg_id,
    p_rec.v.lyg_pavadinimas,
    p_rec.v.lyg_org_id,
    p_rec.v.lyg_kodas,
    p_rec.v.lyg_adresas);
  post_process('INS', p_rec.i, p_rec.v, null);
end;

procedure maintain(
  p_ins in boolean,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val) is
begin
  if p_ins then
/*
  Inicializacija
*/
    if not p_ind.lyg_id or p_new.lyg_id is null then
      p_new.lyg_id := get_lyg_id;
      p_ind.lyg_id := true;
    end if;
  else
/*
  Nekei?iam¸ stulpeli¸ (jei reik?m? netu??ia) patikrinimai
*/
    if p_ind.lyg_id and p_old.lyg_id is not null then
      p_ind.lyg_id := false;
    end if;
  end if;

/*
  "Sen¸" reik?mi¸ perk?limas
*/
  if not p_ind.lyg_id then
    p_new.lyg_id := p_old.lyg_id;
  elsif nvl(p_new.lyg_id, p_old.lyg_id) is null or
     p_new.lyg_id = p_old.lyg_id then
    p_ind.lyg_id := false;
  end if;
  if not p_ind.lyg_pavadinimas then
    p_new.lyg_pavadinimas := p_old.lyg_pavadinimas;
  elsif nvl(p_new.lyg_pavadinimas, p_old.lyg_pavadinimas) is null or
     p_new.lyg_pavadinimas = p_old.lyg_pavadinimas then
    p_ind.lyg_pavadinimas := false;
  end if;
  if not p_ind.lyg_org_id then
    p_new.lyg_org_id := p_old.lyg_org_id;
  elsif nvl(p_new.lyg_org_id, p_old.lyg_org_id) is null or
     p_new.lyg_org_id = p_old.lyg_org_id then
    p_ind.lyg_org_id := false;
  end if;
  if not p_ind.lyg_kodas then
    p_new.lyg_kodas := p_old.lyg_kodas;
  elsif nvl(p_new.lyg_kodas, p_old.lyg_kodas) is null or
     p_new.lyg_kodas = p_old.lyg_kodas then
    p_ind.lyg_kodas := false;
  end if;
  if not p_ind.lyg_adresas then
    p_new.lyg_adresas := p_old.lyg_adresas;
  elsif nvl(p_new.lyg_adresas, p_old.lyg_adresas) is null or
     p_new.lyg_adresas = p_old.lyg_adresas then
    p_ind.lyg_adresas := false;
  end if;

/*
  "Prie?i?ra"
*/
  if p_ind.lyg_pavadinimas then
    p_new.lyg_pavadinimas := trim(p_new.lyg_pavadinimas);
  end if;
  if p_ind.lyg_kodas then
    p_new.lyg_kodas := trim(p_new.lyg_kodas);
  end if;
  if p_ind.lyg_adresas then
    p_new.lyg_adresas := trim(p_new.lyg_adresas);
  end if;
end;

procedure post_process(
  p_operation in varchar2,
  p_ind in t_ind,
  p_new in t_val,
  p_old in t_val) is
  l_api_active boolean;
begin
  if p_operation in ('INS', 'UPD') then
    null;
  end if;
end;

procedure pre_process(
  p_operation in varchar2,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val) is
  l_api_active boolean;
begin
  if p_operation in ('INS', 'UPD') then
    maintain(p_operation = 'INS', p_ind, p_new, p_old);
  end if;
end;

function sel(
  p_id in number,
  p_raise_error in boolean default true) return t_rec is
  l_result t_rec;
begin
  get(p_id, l_result.v, p_raise_error);
  return l_result;
end;

procedure upd(
  p_id in number,
  p_rec in out nocopy t_rec,
  p_raise_error in boolean default true) is
  l_old t_val;
begin
  get(p_id, l_old, p_raise_error);
  if l_old.lyg_id is not null then
    pre_process('UPD', p_rec.i, p_rec.v, l_old);
    update rdm_lygmenys
    set
      lyg_id = p_rec.v.lyg_id,
      lyg_pavadinimas = p_rec.v.lyg_pavadinimas,
      lyg_org_id = p_rec.v.lyg_org_id,
      lyg_kodas = p_rec.v.lyg_kodas,
      lyg_adresas = p_rec.v.lyg_adresas
    where
      lyg_id = p_id;
    if sql%rowcount != 0 then
      post_process('UPD', p_rec.i, p_rec.v, l_old);
    else
      if p_raise_error then
        raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.upd');
      end if;
    end if;
  end if;
end;

procedure synchronize(
  p_id in number,
  p_org_id in number) is

  cursor c(
    p_id in number) is
  select
    lyg_id
  from
    rdm_lygmenys
  where
    lyg_id = p_id;

  l_lyg_id number;
begin
  if p_id is null then
    plog.error('Nenurodytas padalinys');
    raise_application_error(-20001, 'Nenurodytas padalinys. Patikrinkite, ar darbuotojas yra priskirtas bent vienam padaliniui.');
  end if;
  open c(p_id);
  fetch c into l_lyg_id;
  close c;
  if l_lyg_id is null then
    insert into rdm_lygmenys values(
      p_id,
      p_id,
      p_org_id,
      p_id,
      null);
  end if;
end;

end;

/
--------------------------------------------------------
--  DDL for Package Body RDM_MIN
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_MIN" is
/*
  Konstantos
*/
PACKAGE_NAME constant varchar2(30) := 'rdm_min';

/*
  I?ankstiniai paprogrami¸ prototipai
*/

procedure get(
  p_id in number,
  p_val in out nocopy t_val,
  p_raise_error in boolean default true);

procedure maintain(
  p_ins in boolean,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val);

procedure post_process(
  p_operation in varchar2,
  p_ind in t_ind,
  p_new in t_val,
  p_old in t_val);

procedure pre_process(
  p_operation in varchar2,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val);

/*
  Paprogram?s
*/

procedure check_delete(
  p_id in number) is
begin
  declare
    cursor c_rdb(
      p_min_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_darbai
    where
      rdb_min_id_dest = p_min_id;
    l_dummy varchar2(1);
  begin
    open c_rdb(p_id);
    fetch c_rdb into l_dummy;
    close c_rdb;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_MIN_FK');
    end if;
  end;
  declare
    cursor c_rdb(
      p_min_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_darbai
    where
      rdb_min_id_opp = p_min_id;
    l_dummy varchar2(1);
  begin
    open c_rdb(p_id);
    fetch c_rdb into l_dummy;
    close c_rdb;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_MIN_OPONUOTI_FK');
    end if;
  end;
  declare
    cursor c_rdb(
      p_min_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_darbai
    where
      rdb_min_id_stud = p_min_id;
    l_dummy varchar2(1);
  begin
    open c_rdb(p_id);
    fetch c_rdb into l_dummy;
    close c_rdb;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_MIN_PRIKLAUSYTI_FK');
    end if;
  end;
  declare
    cursor c_rdb(
      p_min_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_darbai
    where
      rdb_min_id_rec = p_min_id;
    l_dummy varchar2(1);
  begin
    open c_rdb(p_id);
    fetch c_rdb into l_dummy;
    close c_rdb;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_MIN_RECENZUOTI_FK');
    end if;
  end;
  declare
    cursor c_prn(
      p_min_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_pranesimai
    where
      prn_min_id = p_min_id;
    l_dummy varchar2(1);
  begin
    open c_prn(p_id);
    fetch c_prn into l_dummy;
    close c_prn;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDM_PRN_MIN_FK');
    end if;
  end;
end;

procedure del(
  p_id in number,
  p_raise_error in boolean default true) is
  l_ind t_ind;
  l_new t_val;
  l_old t_val;
begin
  get(p_id, l_old, p_raise_error);
  if l_old.min_id is not null then
    pre_process('DEL', l_ind, l_new, l_old);
    delete /*+first_rows(1)*/ rdm_nariai
    where
      min_id = p_id;
    if sql%rowcount != 0 then
      post_process('DEL', l_ind, l_new, l_old);
    else
      if p_raise_error then
        raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.del');
      end if;
    end if;
  end if;
end;

function find_uk(
  p_el_pastas in varchar2,
  p_raise_error in boolean default true) return rdm_min.t_val is

  cursor c_tab(
    p_el_pastas in varchar2) is
  select /*+first_rows(1)*/
    *
  from
    rdm_nariai
  where
    min_el_pastas = p_el_pastas;
  l_result rdm_min.t_val;
begin
  open c_tab(
    p_el_pastas);
  fetch c_tab into l_result;
  close c_tab;
  if l_result.min_id is null and p_raise_error then
    raise_application_error(-20001, 'RDM_MIN_UK');
  end if;
  return l_result;
end;

procedure get(
  p_id in number,
  p_val in out nocopy t_val,
  p_raise_error in boolean default true) is
  cursor c(
    p_id in number) is
  select /*+first_rows(1)*/
    min_id,
    min_tipas,
    min_el_pastas,
    min_vardas,
    min_org_id,
    min_pavarde,
    min_lyg_id
  from
    rdm_nariai
  where
    min_id = p_id;
begin
  open c(p_id);
  fetch c into p_val;
  close c;
  if p_val.min_id is null and p_raise_error then
    raise_application_error(-200001, PACKAGE_NAME||'.'||TABLE_NAME||'.get');
  end if;
end;

function get_min_id return number is
  cursor c is
  select
    rdm_min_id#.nextval
  from
    dual;
  l_result number;
begin
  open c;
  fetch c into l_result;
  close c;
  return l_result;
end;

procedure ins(
  p_rec in out nocopy t_rec) is
begin
  pre_process('INS', p_rec.i, p_rec.v, null);
  insert into rdm_nariai(
    min_id,
    min_tipas,
    min_el_pastas,
    min_vardas,
    min_org_id,
    min_pavarde,
    min_lyg_id)
  values(
    p_rec.v.min_id,
    p_rec.v.min_tipas,
    p_rec.v.min_el_pastas,
    p_rec.v.min_vardas,
    p_rec.v.min_org_id,
    p_rec.v.min_pavarde,
    p_rec.v.min_lyg_id);
  post_process('INS', p_rec.i, p_rec.v, null);
end;

function has_child_to_invalidate(
  p_id in number) return boolean is
  l_dummy varchar2(1);
begin
  return true;
end;

procedure maintain(
  p_ins in boolean,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val) is
begin
  if p_ins then
/*
  Inicializacija
*/
    if not p_ind.min_id or p_new.min_id is null then
      p_new.min_id := get_min_id;
      p_ind.min_id := true;
    end if;
  else
/*
  Nekei?iam¸ stulpeli¸ (jei reik?m? netu??ia) patikrinimai
*/
    if p_ind.min_id and p_old.min_id is not null then
      p_ind.min_id := false;
    end if;
    if p_ind.min_tipas and p_old.min_tipas is not null then
      p_ind.min_tipas := false;
    end if;
  end if;

/*
  "Sen¸" reik?mi¸ perk?limas
*/
  if not p_ind.min_id then
    p_new.min_id := p_old.min_id;
  elsif nvl(p_new.min_id, p_old.min_id) is null or
     p_new.min_id = p_old.min_id then
    p_ind.min_id := false;
  end if;
  if not p_ind.min_tipas then
    p_new.min_tipas := p_old.min_tipas;
  elsif nvl(p_new.min_tipas, p_old.min_tipas) is null or
     p_new.min_tipas = p_old.min_tipas then
    p_ind.min_tipas := false;
  end if;
  if not p_ind.min_el_pastas then
    p_new.min_el_pastas := p_old.min_el_pastas;
  elsif nvl(p_new.min_el_pastas, p_old.min_el_pastas) is null or
     p_new.min_el_pastas = p_old.min_el_pastas then
    p_ind.min_el_pastas := false;
  end if;
  if not p_ind.min_vardas then
    p_new.min_vardas := p_old.min_vardas;
  elsif nvl(p_new.min_vardas, p_old.min_vardas) is null or
     p_new.min_vardas = p_old.min_vardas then
    p_ind.min_vardas := false;
  end if;
  if not p_ind.min_org_id then
    p_new.min_org_id := p_old.min_org_id;
  elsif nvl(p_new.min_org_id, p_old.min_org_id) is null or
     p_new.min_org_id = p_old.min_org_id then
    p_ind.min_org_id := false;
  end if;
  if not p_ind.min_pavarde then
    p_new.min_pavarde := p_old.min_pavarde;
  elsif nvl(p_new.min_pavarde, p_old.min_pavarde) is null or
     p_new.min_pavarde = p_old.min_pavarde then
    p_ind.min_pavarde := false;
  end if;
  if not p_ind.min_lyg_id then
    p_new.min_lyg_id := p_old.min_lyg_id;
  elsif nvl(p_new.min_lyg_id, p_old.min_lyg_id) is null or
     p_new.min_lyg_id = p_old.min_lyg_id then
    p_ind.min_lyg_id := false;
  end if;

/*
  "Prie?i?ra"
*/
  if p_ind.min_tipas then
    p_new.min_tipas := trim(p_new.min_tipas);
  end if;
  if p_ind.min_el_pastas then
    p_new.min_el_pastas := trim(p_new.min_el_pastas);
  end if;
  if p_ind.min_vardas then
    p_new.min_vardas := trim(p_new.min_vardas);
  end if;
  if p_ind.min_pavarde then
    p_new.min_pavarde := trim(p_new.min_pavarde);
  end if;
end;

procedure post_process(
  p_operation in varchar2,
  p_ind in t_ind,
  p_new in t_val,
  p_old in t_val) is
  l_api_active boolean;
begin
  if p_operation in ('INS', 'UPD') then
    null;
  end if;
end;

procedure pre_process(
  p_operation in varchar2,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val) is
  l_api_active boolean;
begin
  if p_operation in ('INS', 'UPD') then
    maintain(p_operation = 'INS', p_ind, p_new, p_old);
  end if;
end;

function sel(
  p_id in number,
  p_raise_error in boolean default true) return t_rec is
  l_result t_rec;
begin
  get(p_id, l_result.v, p_raise_error);
  return l_result;
end;

procedure upd(
  p_id in number,
  p_rec in out nocopy t_rec,
  p_raise_error in boolean default true) is
  l_old t_val;
begin
  get(p_id, l_old, p_raise_error);
  if l_old.min_id is not null then
    pre_process('UPD', p_rec.i, p_rec.v, l_old);
    update rdm_nariai
    set
      min_id = p_rec.v.min_id,
      min_tipas = p_rec.v.min_tipas,
      min_el_pastas = p_rec.v.min_el_pastas,
      min_vardas = p_rec.v.min_vardas,
      min_org_id = p_rec.v.min_org_id,
      min_pavarde = p_rec.v.min_pavarde,
      min_lyg_id = p_rec.v.min_lyg_id
    where
      min_id = p_id;
    if sql%rowcount != 0 then
      post_process('UPD', p_rec.i, p_rec.v, l_old);
    else
      if p_raise_error then
        raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.upd');
      end if;
    end if;
  end if;
end;

procedure synchronize(
  p_id in number,
  p_email in varchar2,
  p_org_id in number,
  p_givenname in varchar2,
  p_surname in varchar2,
  p_affiliation in varchar2) is

  cursor c(
    p_el_pastas in varchar2) is
  select
    min_id
  from
    rdm_nariai
  where
    min_el_pastas = p_el_pastas;

  l_min_id number;
begin
  open c(p_email);
  fetch c into l_min_id;
  close c;
  if l_min_id is null then
    insert into rdm_nariai values(
      p_id,
      p_affiliation,
      p_email,
      p_givenname,
      p_org_id,
      p_surname,
      null);
  elsif p_id != l_min_id then
    update rdm_nariai set
      min_id = p_id,
      min_vardas = p_givenname,
      min_pavarde = p_surname,
      min_tipas = p_affiliation
    where
      min_id = l_min_id;

    update rdm_darbai set
      rdb_min_id_dest = p_id
    where
      rdb_min_id_dest = l_min_id;

    update rdm_darbai set
      rdb_min_id_rec = p_id
    where
      rdb_min_id_rec = l_min_id;

    update rdm_darbai set
      rdb_min_id_opp = p_id
    where
      rdb_min_id_opp = l_min_id;

    update rdm_darbai set
      rdb_min_id_stud = p_id
    where
      rdb_min_id_stud = l_min_id;
  end if;
end;

end;

/
--------------------------------------------------------
--  DDL for Package Body RDM_ORG
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_ORG" is
/*
  Konstantos
*/
PACKAGE_NAME constant varchar2(30) := 'rdm_org';

/*
  I?ankstiniai paprogrami¸ prototipai
*/

procedure get(
  p_id in number,
  p_val in out nocopy t_val,
  p_raise_error in boolean default true);

procedure maintain(
  p_ins in boolean,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val);

procedure post_process(
  p_operation in varchar2,
  p_ind in t_ind,
  p_new in t_val,
  p_old in t_val);

procedure pre_process(
  p_operation in varchar2,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val);

/*
  Paprogram?s
*/

procedure check_delete(
  p_id in number) is
begin
  declare
    cursor c_min(
      p_org_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_nariai
    where
      min_org_id = p_org_id;
    l_dummy varchar2(1);
  begin
    open c_min(p_id);
    fetch c_min into l_dummy;
    close c_min;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_MIN_ORG_FK');
    end if;
  end;
  declare
    cursor c_rdb(
      p_org_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_darbai
    where
      rdb_org_id = p_org_id;
    l_dummy varchar2(1);
  begin
    open c_rdb(p_id);
    fetch c_rdb into l_dummy;
    close c_rdb;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDB_ORG_FK');
    end if;
  end;
  declare
    cursor c_lyg(
      p_org_id in number) is
    select /*+first_rows(1)*/
      '1'
    from
      rdm_lygmenys
    where
      lyg_org_id = p_org_id;
    l_dummy varchar2(1);
  begin
    open c_lyg(p_id);
    fetch c_lyg into l_dummy;
    close c_lyg;
    if l_dummy is not null then
      raise_application_error(-20001, 'RDM_LYG_ORG_FK');
    end if;
  end;
end;

procedure del(
  p_id in number,
  p_raise_error in boolean default true) is
  l_ind t_ind;
  l_new t_val;
  l_old t_val;
begin
  get(p_id, l_old, p_raise_error);
  if l_old.org_id is not null then
    pre_process('DEL', l_ind, l_new, l_old);
    delete /*+first_rows(1)*/ rdm_organizacijos
    where
      org_id = p_id;
    if sql%rowcount != 0 then
      post_process('DEL', l_ind, l_new, l_old);
    else
      if p_raise_error then
        raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.del');
      end if;
    end if;
  end if;
end;

function find_uk(
  p_kodas in varchar2,
  p_raise_error in boolean default true) return rdm_org.t_val is

  cursor c_tab(
    p_kodas in varchar2) is
  select /*+first_rows(1)*/
    *
  from
    rdm_organizacijos
  where
    org_kodas = p_kodas;
  l_result rdm_org.t_val;
begin
  open c_tab(
    p_kodas);
  fetch c_tab into l_result;
  close c_tab;
  if l_result.org_id is null and p_raise_error then
      raise_application_error(-20001, 'RDM_ORG');
  end if;
  return l_result;
end;

procedure get(
  p_id in number,
  p_val in out nocopy t_val,
  p_raise_error in boolean default true) is
  cursor c(
    p_id in number) is
  select /*+first_rows(1)*/
    org_id,
    org_pavadinimas,
    org_kodas,
    org_adresas
  from
    rdm_organizacijos
  where
    org_id = p_id;
begin
  open c(p_id);
  fetch c into p_val;
  close c;
  if p_val.org_id is null and p_raise_error then
    raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.get');
  end if;
end;

function get_org_id return number is
  cursor c is
  select
    rdm_org_id#.nextval
  from
    dual;
  l_result number;
begin
  open c;
  fetch c into l_result;
  close c;
  return l_result;
end;

procedure ins(
  p_rec in out nocopy t_rec) is
begin
  pre_process('INS', p_rec.i, p_rec.v, null);
  insert into rdm_organizacijos(
    org_id,
    org_pavadinimas,
    org_kodas,
    org_adresas)
  values(
    p_rec.v.org_id,
    p_rec.v.org_pavadinimas,
    p_rec.v.org_kodas,
    p_rec.v.org_adresas);
  post_process('INS', p_rec.i, p_rec.v, null);
end;

procedure maintain(
  p_ins in boolean,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val) is
begin
  if p_ins then
/*
  Inicializacija
*/
    if not p_ind.org_id or p_new.org_id is null then
      p_new.org_id := get_org_id;
      p_ind.org_id := true;
    end if;
  else
/*
  Nekei?iam¸ stulpeli¸ (jei reik?m? netu??ia) patikrinimai
*/
    if p_ind.org_id and p_old.org_id is not null then
      p_ind.org_id := false;
    end if;
  end if;

/*
  "Sen¸" reik?mi¸ perk?limas
*/
  if not p_ind.org_id then
    p_new.org_id := p_old.org_id;
  elsif nvl(p_new.org_id, p_old.org_id) is null or
     p_new.org_id = p_old.org_id then
    p_ind.org_id := false;
  end if;
  if not p_ind.org_pavadinimas then
    p_new.org_pavadinimas := p_old.org_pavadinimas;
  elsif nvl(p_new.org_pavadinimas, p_old.org_pavadinimas) is null or
     p_new.org_pavadinimas = p_old.org_pavadinimas then
    p_ind.org_pavadinimas := false;
  end if;
  if not p_ind.org_kodas then
    p_new.org_kodas := p_old.org_kodas;
  elsif nvl(p_new.org_kodas, p_old.org_kodas) is null or
     p_new.org_kodas = p_old.org_kodas then
    p_ind.org_kodas := false;
  end if;
  if not p_ind.org_adresas then
    p_new.org_adresas := p_old.org_adresas;
  elsif nvl(p_new.org_adresas, p_old.org_adresas) is null or
     p_new.org_adresas = p_old.org_adresas then
    p_ind.org_adresas := false;
  end if;

/*
  "Prie?i?ra"
*/
  if p_ind.org_pavadinimas then
    p_new.org_pavadinimas := trim(p_new.org_pavadinimas);
  end if;
  if p_ind.org_kodas then
    p_new.org_kodas := trim(p_new.org_kodas);
  end if;
  if p_ind.org_adresas then
    p_new.org_adresas := trim(p_new.org_adresas);
  end if;
end;

procedure post_process(
  p_operation in varchar2,
  p_ind in t_ind,
  p_new in t_val,
  p_old in t_val) is
  l_api_active boolean;
begin
  if p_operation in ('INS', 'UPD') then
    null;
  end if;
end;

procedure pre_process(
  p_operation in varchar2,
  p_ind in out nocopy t_ind,
  p_new in out nocopy t_val,
  p_old in t_val) is
  l_api_active boolean;
begin
  if p_operation in ('INS', 'UPD') then
    maintain(p_operation = 'INS', p_ind, p_new, p_old);
  end if;
end;

function sel(
  p_id in number,
  p_raise_error in boolean default true) return t_rec is
  l_result t_rec;
begin
  get(p_id, l_result.v, p_raise_error);
  return l_result;
end;

procedure upd(
  p_id in number,
  p_rec in out nocopy t_rec,
  p_raise_error in boolean default true) is
  l_old t_val;
begin
  get(p_id, l_old, p_raise_error);
  if l_old.org_id is not null then
    pre_process('UPD', p_rec.i, p_rec.v, l_old);
    update rdm_organizacijos
    set
      org_id = p_rec.v.org_id,
      org_pavadinimas = p_rec.v.org_pavadinimas,
      org_kodas = p_rec.v.org_kodas,
      org_adresas = p_rec.v.org_adresas
    where
      org_id = p_id;
    if sql%rowcount != 0 then
      post_process('UPD', p_rec.i, p_rec.v, l_old);
    else
      if p_raise_error then
        raise_application_error(-20001, PACKAGE_NAME||'.'||TABLE_NAME||'.upd');
      end if;
    end if;
  end if;
end;

procedure synchronize(
  p_kodas in varchar2) is

  l_rdm_rec rdm_org.t_rec;
begin
  l_rdm_rec.v := find_uk(p_kodas, false);
  if l_rdm_rec.v.org_id is null then
    l_rdm_rec.v.org_id := null;
    l_rdm_rec.v.org_pavadinimas := p_kodas;
    l_rdm_rec.v.org_kodas := p_kodas;
    l_rdm_rec.v.org_adresas := null;
    l_rdm_rec.i.org_id := true;
    l_rdm_rec.i.org_pavadinimas := true;
    l_rdm_rec.i.org_kodas := true;
    l_rdm_rec.i.org_adresas := true;
    ins(l_rdm_rec);
  end if;
end;

procedure synchronize(
  p_id in number,
  p_pavadinimas in varchar2) is

  cursor c(
    p_id in number) is
  select
    org_id
  from
    rdm_organizacijos
  where
    org_id = p_id;

  l_org_id number;
begin
  if p_id is null then
    plog.error('Nenurodyta organizacija');
    raise_application_error(-20001, 'Nenurodyta organizacija. Patikrinkite, ar darbuotojas yra priskirtas organizacijai.');
  end if;
  open c(p_id);
  fetch c into l_org_id;
  close c;
  if l_org_id is null then
    insert into rdm_organizacijos values(
      p_id,
      p_pavadinimas,
      p_id,
      null);
  end if;
end;

end;

/
--------------------------------------------------------
--  DDL for Package Body RDM_PRM
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_PRM" is

function find(
  p_name in varchar2) return t_val is

  cursor c_prm(
    p_name in varchar2) is
  select
    *
  from
    rdm_parametrai
  where
    upper(prm_pavadinimas) = p_name;

  l_result t_val;
begin
/*
  Ie?kome sistemos parametro
*/
  open c_prm(upper(p_name));
  fetch c_prm into l_result;
  close c_prm;

/*
  Gr??iname sistemos parametro atribut rinkin?
*/
  return l_result;
end;

function get_value(
  p_name in varchar2,
  p_default_value in varchar2 default null) return varchar2 is

  l_prm t_val;
begin
/*
  Ie?kome nurodyto sistemos parametro
*/
  l_prm := find(p_name);

/*
  Gra?iname parametro reik?m? arba, jei pastaroji neapibr??ta, nurodyt? standartin?
  parametro reik?m?
*/
  return nvl(l_prm.prm_verte, p_default_value);
end;

/*dml operacija - insert*/
procedure ins(
  p_prm_pavadinimas in varchar2,
  p_prm_verte in varchar2,
  p_prm_aprasymas in varchar2 default null) is
begin
  insert into rdm_parametrai(
    prm_id,
    prm_pavadinimas,
    prm_verte,
    prm_aprasymas)
  values(
    rdm_prm_id#.nextval,
    p_prm_pavadinimas,
    p_prm_verte,
    p_prm_aprasymas);
end;

end RDM_PRM;

/
--------------------------------------------------------
--  DDL for Package Body RDM_RDB
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_RDB" is

  /*Proced?r? ir funkcij? deklaracijos*/
procedure show_blob(
  p_blob in blob);

  /*proced?r? ir funkcij? paprogram?s*/
procedure get_content(
  p_rdb_id in number) is

  cursor c_rdb(
    p_rdb_id in number) is
  select
    rdb_turinys
  from
    rdm_darbai
  where
    rdb_id = p_rdb_id;

  l_blob blob;
begin
  open c_rdb(p_rdb_id);
  fetch c_rdb into l_blob;
  close c_rdb;

  show_blob(l_blob);
end;

procedure show_blob(
  p_blob in blob) is

  c_max_length constant binary_integer := 8000;
  l_clob clob;
  l_clob_length integer;
  l_buffer varchar2(32000);
  l_chunk_count integer;
  l_index integer;
  l_read binary_integer;
begin
  l_clob := blobtoclob(p_blob);
  l_clob_length := dbms_lob.getlength(l_clob);
  --owa_util.mime_header( 'application/pdf' );
  if l_clob_length < c_max_length then
    l_read := l_clob_length;
    dbms_lob.read(l_clob, l_read, 1, l_buffer);
    htp.p(l_buffer);
  else
    l_read := c_max_length;
    l_chunk_count := floor(l_clob_length/c_max_length);
    l_index := 0;
    while l_index < l_chunk_count loop
      dbms_lob.read(l_clob, l_read, (l_index*c_max_length+1), l_buffer);
      htp.p(l_buffer);
      l_read := c_max_length;
      l_index := l_index + 1;
    end loop;
    l_read := l_clob_length - l_chunk_count*c_max_length;
    if l_read > 0 then
      dbms_lob.read(l_clob, l_read, (l_index*c_max_length+1), l_buffer);
      htp.p(l_buffer);
    end if;
  end if;
  dbms_lob.freetemporary(l_clob);
exception
  when others then
    dbms_lob.freetemporary(l_clob);
    raise;
end;

function blobtoclob(
  p_blob in blob) return clob is

  l_clob_loc clob;
  l_amount integer := DBMS_LOB.LOBMAXSIZE;
  l_dest_offset integer := 1;
  l_src_offset integer := 1;
  l_blob_csid number := DBMS_LOB.default_csid;
  l_lang_context integer := DBMS_LOB.default_lang_ctx;
  l_warning integer := DBMS_LOB.no_warning;
begin
  dbms_lob.createtemporary(l_clob_loc, false);
  dbms_lob.convertToClob(l_clob_loc,
                         p_blob,
                         l_amount,
                         l_dest_offset,
                         l_src_offset,
                         l_blob_csid,
                         l_lang_context,
                         l_warning);
  return l_clob_loc;
exception
  when others then
    dbms_lob.freetemporary(l_clob_loc);
end;

function clobtoblob(
  p_clob in clob) return blob is

  l_blob_loc blob;
  l_amount integer := DBMS_LOB.LOBMAXSIZE;
  l_dest_offset integer := 1;
  l_src_offset integer := 1;
  l_blob_csid number := DBMS_LOB.default_csid;
  l_lang_context integer := DBMS_LOB.default_lang_ctx;
  l_warning integer := DBMS_LOB.no_warning;
begin
  dbms_lob.createtemporary(l_blob_loc, false);
  dbms_lob.convertToBlob(l_blob_loc,
                         p_clob,
                         l_amount,
                         l_dest_offset,
                         l_src_offset,
                         l_blob_csid,
                         l_lang_context,
                         l_warning);
  return l_blob_loc;
exception
  when others then
    dbms_lob.freetemporary(l_blob_loc);
end;

procedure add(
  p_rec in t_rec) is

begin
  insert into rdm_darbai(
    rdb_id,
    rdb_failo_pavadinimas,
    rdb_darbo_pavadinimas,
    rdb_ikelimo_data,
    rdb_metai,
    rdb_min_id_stud,
    rdb_min_id_dest,
    rdb_org_id,
    rdb_lyg_id,
    rdb_mime_tipas,
    rdb_turinys,
    rdb_statusas,
    rdb_min_id_rec,
    rdb_min_id_opp,
    rdb_epas_oa_id,
    rdb_panasumo_procentas,
    rdb_nebetikrinti_procento,
    rdb_pakeitimo_data)
  values(
    rdm_rdb_id#.nextval,
    p_rec.rdb_failo_pavadinimas,
    p_rec.rdb_darbo_pavadinimas,
    p_rec.rdb_ikelimo_data,
    p_rec.rdb_metai,
    p_rec.rdb_min_id_stud,
    p_rec.rdb_min_id_dest,
    p_rec.rdb_org_id,
    p_rec.rdb_lyg_id,
    p_rec.rdb_mime_tipas,
    p_rec.rdb_turinys,
    p_rec.rdb_statusas,
    p_rec.rdb_min_id_rec,
    p_rec.rdb_min_id_opp,
    p_rec.rdb_epas_oa_id,
    p_rec.rdb_panasumo_procentas,
    p_rec.rdb_nebetikrinti_procento,
    p_rec.rdb_pakeitimo_data);
end;

procedure upd(
  p_rec in t_rec) is
begin
  update rdm_darbai set
    rdb_statusas = p_rec.rdb_statusas,
    rdb_epas_oa_id = p_rec.rdb_epas_oa_id
  where
    rdb_id = p_rec.rdb_id;

end;

procedure del(
  p_rdb_id in number) is
begin
  delete from rdm_darbai
  where
    rdb_id = p_rdb_id;
end;

procedure set_sim_score(
  p_rdb_id in number,
  p_sim_score in number,
  p_nebetikrinti_procento in number default 1) is
begin
  update rdm_darbai set
    rdb_panasumo_procentas = p_sim_score,
    rdb_nebetikrinti_procento = p_nebetikrinti_procento
  where
    rdb_id = p_rdb_id;
end;

function get_rgb(
  p_similarity_score in number) return varchar2 is

  l_result varchar2(50);

  color_interval constant number := 360;
  l_color_score number;
  l_color_r number(10) := 20;
  l_color_g number(10) := 200;
  l_color_b number(10) := 20;
begin
  if p_similarity_score = 0 then
    l_color_score := 0;
  else
    l_color_score := floor((360*(1/POWER(p_similarity_score,(-1/4))))/(1/POWER(100,(-1/4))));
  end if;

  if l_color_score > color_interval/2 then
    l_color_r := 200;
    l_color_g := l_color_g - (l_color_score - color_interval/2);
  else
    l_color_r := l_color_r + l_color_score;
  end if;

  l_result := 'rgb(' || l_color_r || ', ' || l_color_g || ', ' || l_color_b || ')';
  return l_result;
end;

end RDM_RDB;

/
--------------------------------------------------------
--  DDL for Package Body RDM_UTL
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_UTL" as

function split(
  p_list varchar2,
  p_delimiter varchar2 default C_SEMICOLON) return t_string_array is

  l_index pls_integer;
  l_array_index pls_integer := 0;
  l_list varchar2(32767) := p_list;
  l_result t_string_array;
begin
  loop
    l_index := instr(l_list, p_delimiter);
    if l_index > 0 then
      l_result(l_array_index) := substr(l_list, 1, l_index-1);
      l_list := substr(l_list, l_index + length(p_delimiter));
      l_array_index := l_array_index + 1;
    else
      l_result(l_array_index) := l_list;
      exit;
    end if;
  end loop;
  return l_result;
end split;

end rdm_utl;

/
--------------------------------------------------------
--  DDL for Package Body RDM_VERSION
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_VERSION" is

/*
  I?ankstin? deklaracija
*/
function is_hot_installed(
  p_application in varchar2,
  p_version in varchar2) return boolean;

/*
  Proced?ra patikrina DB baz?s objekt diegimo versij?
  %param p_application Aplikacijos trumpinys.
  %param p_version Dabartin? aplikacijos versija.
  %param p_next_version Diegiama aplikacijos versija.
  %param p_default Versijos reik?m? pagal nutyl?jim?.
*/
procedure check_version(
  p_application in varchar2,
  p_version in varchar2,
  p_next_version in varchar2,
  p_default in varchar2 default '0.0') is

  cursor c_rel(p_application in varchar2) is
  select
    ver_label
  from
    rdm_versions
  where
    ver_application = p_application
  order by
    ver_id desc;

  l_current_version rdm_versions.ver_label%type;
begin
  open c_rel(p_application);
  fetch c_rel into l_current_version;
  close c_rel;

  if nvl(l_current_version, p_default) not like p_version||'%' then
    raise_application_error(-20000, 'Sistema: "'||p_application||'" dabartin? versija: "'||nvl(l_current_version, p_default)||'". Pakeitim diegimui reikalinga versija: "'||p_version||'"');
  else
    dbms_output.put_line('Sistema: "'||p_application||'" dabartin? versija: "'||nvl(l_current_version, p_default)||'"');
  end if;

  if is_hot_installed(p_application, p_next_version) then
    set_version(
      p_application,
      p_next_version);

    raise_application_error(-20000, 'Versija: "'||p_next_version||'" sudiegta kaip "kar?tas" pakeitim rinkinys. Pakartotinas diegimas sustabdomas.');
  end if;
end;

/*
  Funkcija patikrina ar versija sudiegta kaip kar?tas pakeitim rinkinys.
*/
function is_hot_installed(
  p_application in varchar2,
  p_version in varchar2) return boolean is

  cursor c_rel(
    p_application in varchar2,
    p_version in varchar2) is
  select
    '1'
  from
    rdm_versions
  where
    ver_application = p_application and
    ver_base_label = p_version;
  l_dummy varchar2(1);
begin
  open c_rel(p_application, p_version);
  fetch c_rel into l_dummy;
  close c_rel;
  return l_dummy is not null;
end;

/*
  Proced?ra nustato aplikacijos versij?.
  %param p_application Aplikacijos trumpinys.
  %param p_version Dabartin? aplikacijos versija.
  %param p_base_version
*/
procedure set_version(
  p_application in varchar2,
  p_version in varchar2,
  p_base_version in varchar2 default null) is

  pragma autonomous_transaction;
begin
  insert into rdm_versions(
    ver_id,
    ver_application,
    ver_install_date,
    ver_label,
    ver_base_label)
  values(
    rdm_ver_id#.nextval,
    p_application,
    sysdate,
    p_version,
    p_base_version);

  dbms_output.put_line('Sistema: "'||p_application||'" nustatyta versija: "'||p_version||'"');

  commit;
end;

end RDM_VERSION;

/
--------------------------------------------------------
--  DDL for Package Body RDM_WS
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "RDM_WS" is

/*i?ansktin?s funkcij ir proced?r deklaracijos*/
function send_doc_to_epas(
  p_id in number) return varchar2;

/*funkcij ir proced?r paprogram?s*/
function get_list_of_documents(
  p_start_date in varchar2) return list_tab pipelined is

  cursor c_doc_list(
    p_start_date in varchar2) is
  select
    rdb_id id,
    rdb_failo_pavadinimas file_name,
    rdb_darbo_pavadinimas title,
    s.min_vardas || ' ' || s.min_pavarde student,
    s.min_id student_id,
    t.min_vardas || ' ' || t.min_pavarde teacher,
    t.min_id teacher_id,
    r.min_vardas || ' ' || r.min_pavarde reviewer,
    r.min_id reviewer_id,
    o.min_vardas || ' ' || o.min_pavarde opponent,
    o.min_id opponent_id,
    sl.lyg_pavadinimas division,
    sl.lyg_id division_id,
    so.org_pavadinimas organization,
    so.org_id organization_id,
    rdb_metai year,
    rdb_statusas status
  from
    rdm_darbai d left outer join /* buvo inner*/
    rdm_nariai s on s.min_id = d.rdb_min_id_stud left outer join /* buvo inner*/
    rdm_lygmenys sl on sl.lyg_id = d.rdb_lyg_id left outer join /* buvo inner*/
    rdm_organizacijos so on so.org_id = d.rdb_org_id left outer join /* buvo inner*/
    rdm_nariai t on t.min_id = d.rdb_min_id_dest left outer join /* buvo inner*/
    rdm_nariai r on r.min_id = d.rdb_min_id_rec left outer join
    rdm_nariai o on o.min_id = d.rdb_min_id_opp
  where
    nvl(rdb_pakeitimo_data, rdb_ikelimo_data) > nvl(to_date(p_start_date, 'YYYY-MM-DD HH24:MI:SS'), to_date('1900-01-01', 'YYYY-MM-DD'));

  l_list_val t_val;
begin
  for c_fetched in c_doc_list(p_start_date)
  loop
    l_list_val.id := c_fetched.id;
    l_list_val.title := c_fetched.title;
    l_list_val.student := c_fetched.student;
    l_list_val.student_id := c_fetched.student_id;
    l_list_val.teacher := c_fetched.teacher;
    l_list_val.teacher_id := c_fetched.teacher_id;
    l_list_val.reviewer := c_fetched.reviewer;
    l_list_val.reviewer_id := c_fetched.reviewer_id;
    l_list_val.opponent := c_fetched.opponent;
    l_list_val.opponent_id := c_fetched.opponent_id;
    l_list_val.division := c_fetched.division;
    l_list_val.division_id := c_fetched.division_id;
    l_list_val.organization := c_fetched.organization;
    l_list_val.organization_id := c_fetched.organization_id;
    l_list_val.year := c_fetched.year;
    l_list_val.status := c_fetched.status;
    l_list_val.source_code := rdm_prm.get_value('source_code');
   pipe row(l_list_val);
  end loop;

  return;
end;

/*suformuojame json, kuriame yra s?ra?as vis dokument, kurios reikia sisti ? EPAS nuolatinio
ra?to darb atspaud DB papildymui, modifikavimui ar trinimui*/
function format_full_list_documents(
  p_start_date in varchar2) return clob is

  cursor c_docs(
    p_start_date in varchar2) is
  select
    *
  from
    table(get_list_of_documents(p_start_date));

  l_json json;
  l_full_json json;
  l_json_list_add json_list; /*pridedam arba modifikuojam dokumen s?ra?as*/
  l_json_list_del json_list; /*trinam dokument s?ra?as*/
  l_source_code varchar2(240) := rdm_prm.get_value('source_code');
  l_clob clob;
  l_date_stamp varchar2(20) := to_char(sysdate, 'YYYY-MM-DD HH24:MI');
begin
  l_json := json(); --fresh json;
  l_full_json := json();
  l_json_list_add := json_list();
  l_json_list_del := json_list();

  for c_doc in c_docs(p_start_date) loop
    l_json := json();
    l_json.put('id', c_doc.id);
    l_json.put('file_name', c_doc.file_name);
    l_json.put('title', c_doc.title);
    l_json.put('student', c_doc.student);
    l_json.put('student_id', c_doc.student_id);
    l_json.put('teacher', c_doc.teacher);
    l_json.put('teacher_id', c_doc.teacher_id);
    l_json.put('reviewer', c_doc.reviewer);
    l_json.put('reviewer_id', c_doc.reviewer_id);
    l_json.put('opponent', c_doc.opponent);
    l_json.put('opponent_id', c_doc.opponent_id);
    l_json.put('division', c_doc.division);
    l_json.put('division_id', c_doc.division_id);
    l_json.put('organization', c_doc.organization);
    l_json.put('organization_id', c_doc.organization_id);
    l_json.put('year', c_doc.year);

    if c_doc.status = 'D' then
      l_json_list_del.append(l_json.to_json_value);
    else
      l_json_list_add.append(l_json.to_json_value);
    end if;
  end loop;

  l_full_json.put('add', l_json_list_add);
  l_full_json.put('delete', l_json_list_del);
  l_json := json();
  l_json.put('docs', l_full_json.to_json_value);
  l_json.put('end_date', l_date_stamp);
  l_json.put('source_code', l_source_code);
 -- l_json.print;
  dbms_lob.createtemporary(l_clob, false);
  l_json.to_clob(l_clob, true);
  return l_clob;
end;

procedure get_oa_id(
  p_doc_id in number) is

  l_rec rdm_rdb.t_rec;
  l_response varchar2(3000);
begin
  rdm_rdb.set_sim_score(p_doc_id, null, 0);

  l_rec.rdb_statusas := 'P';
  l_rec.rdb_id := p_doc_id;
  /* siunčiame dokumentą į EPAS. Grįžta ataskaitos ID */
  l_response := rdm_ws.send_doc_to_epas(p_doc_id);
  l_rec.rdb_epas_oa_id := to_number(l_response);
  if l_rec.rdb_epas_oa_id < 1 then
    l_rec.rdb_epas_oa_id := null;
    l_rec.rdb_statusas := 'K';
  end if;
  rdm_rdb.upd(l_rec);
exception
  when others then
    plog.warn('Nepavyko gauti originalumo ataskaitos ID. DOC_ID:'||p_doc_id);
    l_rec.rdb_epas_oa_id := null;
    l_rec.rdb_statusas := 'I';
    rdm_rdb.upd(l_rec);
end;

function send_doc_to_epas(
  p_id in number) return varchar2 is

  cursor c_docs(
    p_id in number) is
  select
    t.*,
    d.rdb_turinys,
    d.rdb_mime_tipas,
    d.rdb_failo_pavadinimas
  from
    table (rdm_ws.get_list_of_documents(null)) t inner join
    rdm_darbai d on d.rdb_id = t.id
  where
    t.ID = p_id;

  e_bad_response exception;
  l_req   utl_http.req;
  l_resp  utl_http.resp;
  l_response varchar2(3000);
  l_value varchar2(1000);
  l_index number;
  l_blob_length number;
  l_buffer blob;
  l_amount pls_integer := 32000;
  l_offset pls_integer := 1;
  l_json json;
  l_url varchar2(1000);
begin
  for cur in c_docs(p_id)
  loop
    l_blob_length := DBMS_LOB.getlength(cur.rdb_turinys);

    l_url := rdm_prm.get_value('DOC_PROC_WS');
    plog.debug('Requesting to:'||l_url);
 
    utl_http.set_body_charset(charset => 'utf-8');
    l_req := utl_http.begin_request(l_url, method => 'POST');

    utl_http.set_header(l_req, 'source_code', rdm_prm.get_value('source_code'));
    utl_http.set_header(l_req, 'id', cur.id);
    plog.debug('id:'||cur.id);
    utl_http.set_header(l_req, 'file_name', utl_url.escape(cur.rdb_failo_pavadinimas, url_charset => 'utf-8'));
    plog.debug('file name:'||cur.rdb_failo_pavadinimas);
    utl_http.set_header(l_req, 'title', utl_url.escape(cur.title, url_charset => 'utf-8'));
    plog.debug('title:'||cur.title);
    utl_http.set_header(l_req, 'author', utl_url.escape(cur.student, url_charset => 'utf-8'));
    plog.debug('student:'||cur.student);
    utl_http.set_header(l_req, 'author_id', cur.student_id);
    plog.debug('student_id:'||cur.student_id);
    utl_http.set_header(l_req, 'teacher', utl_url.escape(cur.teacher, url_charset => 'utf-8'));
    plog.debug('teacher:'||cur.teacher);
    utl_http.set_header(l_req, 'teacher_id', cur.teacher_id);
    plog.debug('teacher_id:'||cur.teacher_id);
    utl_http.set_header(l_req, 'reviewer', utl_url.escape(cur.reviewer, url_charset => 'utf-8'));
    plog.debug('reviewer:'||cur.reviewer);
    utl_http.set_header(l_req, 'reviewer_id', cur.reviewer_id);
    plog.debug('reviewer_id:'||cur.reviewer_id);
    utl_http.set_header(l_req, 'opponent', utl_url.escape(cur.opponent, url_charset => 'utf-8'));
    plog.debug('opponent:'||cur.opponent);
    utl_http.set_header(l_req, 'opponent_id', cur.opponent_id);
    plog.debug('opponent_id:'||cur.opponent_id);
    utl_http.set_header(l_req, 'division', utl_url.escape(cur.division, url_charset => 'utf-8'));
    plog.debug('division:'||cur.division);
    utl_http.set_header(l_req, 'division_id', cur.division_id);
    plog.debug('division_id:'||cur.division_id);
    utl_http.set_header(l_req, 'organization', utl_url.escape(cur.organization, url_charset => 'utf-8'));
    plog.debug('organization:'||cur.organization);
    utl_http.set_header(l_req, 'organization_id', cur.organization_id);
    plog.debug('organization_id:'||cur.organization_id);
    utl_http.set_header(l_req, 'year', cur.year);
    plog.debug('year:'||cur.year);
    utl_http.set_header(l_req, 'Content-Type', cur.rdb_mime_tipas);
    plog.debug('content-type:'||cur.rdb_mime_tipas);

   if l_blob_length <= 32767
   then /*jei turinys ma?esnis, nei 32767, privalome nurodyti header'? Content-Length*/
     utl_http.set_header(l_req, 'Content-Length', l_blob_length);
     utl_http.write_raw(l_req, cur.rdb_turinys);
   elsif l_blob_length > 32767
   then /*jei turinys ilgesnis, nei 32767, privalome sisti blob'? dalimis ir nurodyti header'? Transfer-Encoding*/
     utl_http.set_header(l_req, 'Transfer-Encoding', 'chunked');
     while (l_offset < l_blob_length)
     loop
       dbms_lob.read(cur.rdb_turinys, l_amount, l_offset, l_buffer);
       utl_http.write_raw(l_req, l_buffer);
       l_offset := l_offset + l_amount;
     end loop;
   end if;

    /* perskaitom atsakymą */
    l_resp := utl_http.get_response(l_req);
    plog.debug('Response code:'||l_resp.status_code);
    if l_resp.status_code != utl_http.HTTP_OK then
      raise e_bad_response;
    end if;
    begin
      loop
        utl_http.read_text(l_resp, l_value);
        l_response :=  l_response || l_value;
      end loop;
      utl_http.end_response(l_resp);
    exception
      when utl_http.END_OF_BODY then
        utl_http.end_response(l_resp);
    end;
    plog.debug('id:'||l_response);

    l_json := json(l_response);
    /* Išsilupame iš jsono ataskaitos ID gautą iš EPAS */
    l_response := json_ext.get_string(l_json, 'oa_id');
  end loop;
  return l_response;
exception
  when e_bad_response then
    plog.error('Unable process response because of HTTP response status');
    return -1;
  when others then
    plog.error(dbms_utility.format_error_stack);
    return -1;
end;

function get_oa_status(
  p_oa_id in number) return varchar2 is

  l_req   utl_http.req;
  l_resp  utl_http.resp;
  l_response varchar2(3000);
  l_value varchar2(1000);
begin
  l_req := utl_http.begin_request(
      url => rdm_prm.get_value('DOC_STATUS_WS') || p_oa_id,
      method => 'GET');
  l_resp := utl_http.get_response(l_req);
  begin
    loop
      utl_http.read_text(l_resp, l_value);
      l_response :=  l_response || l_value;
    end loop;
    utl_http.end_response(l_resp);
  exception
    when utl_http.END_OF_BODY then
      utl_http.end_response(l_resp);
  end;
  return l_response;
end;

function get_sim_score(
  p_oa_id in number) return t_similarity_score_ws is

  l_json json;
  l_req   utl_http.req;
  l_resp  utl_http.resp;
  l_response varchar2(3000);
  l_value varchar2(1000);
  l_dom_document dbms_xmldom.DOMDocument;
  l_children dbms_xmldom.DOMNodeList;
  l_child dbms_xmldom.DOMNode;
  l_result t_similarity_score_ws;
begin
  l_req := utl_http.begin_request(
      url => replace(rdm_prm.get_value('SIMSCORE_WS'), '{id}', p_oa_id),
      method => 'GET');
  l_resp := utl_http.get_response(l_req);
  begin
    loop
      utl_http.read_text(l_resp, l_value);
      l_response :=  l_response || l_value;
    end loop;
    utl_http.end_response(l_resp);
  exception
    when utl_http.END_OF_BODY then
      utl_http.end_response(l_resp);
  end;

  l_json := json(l_response);
  l_result.similarity_score := json_ext.get_number(l_json, 'similarity_score');
  l_result.report_status := json_ext.get_number(l_json, 'report_status');
  l_result.report_text := json_ext.get_string(l_json, 'report_text');

 return l_result;
end;

end RDM_WS;

/
--------------------------------------------------------
--  DDL for Package Body SHIBBOLETH
--------------------------------------------------------

  CREATE OR REPLACE  PACKAGE BODY "SHIBBOLETH" as

/*
  Funkcija užpildo ir grąžina Shibboleth perduotų atributų reikšmes. Į DB atributai
  atkeliauja kaip CGI aplinkos kintamieji.
  %return t_shibboleth_attributes tipo kintamasis užpildytas iš SSO gautomis atributų reikšmėmis.
*/
function get_attributes return t_attributes is
  l_shibb_attrs t_attributes;
begin
  plog.debug('Content-Type:'||owa_util.get_cgi_env('Content-Type'));

  l_shibb_attrs.uid := owa_util.get_cgi_env('uid');
  plog.debug('uid:'||l_shibb_attrs.uid);
  l_shibb_attrs.commonname := owa_util.get_cgi_env('commonname');
  plog.debug('commonname:'||l_shibb_attrs.commonname);
  l_shibb_attrs.surname := owa_util.get_cgi_env('surname');
  plog.debug('surname:'||l_shibb_attrs.surname);
  l_shibb_attrs.givenname := owa_util.get_cgi_env('givenname');
  plog.debug('givenname:'||l_shibb_attrs.givenname);
  l_shibb_attrs.displayname := owa_util.get_cgi_env('displayname');
  plog.debug('displayname:'||l_shibb_attrs.displayname);
  l_shibb_attrs.email := owa_util.get_cgi_env('email');
  plog.debug('email:'||l_shibb_attrs.email);
  l_shibb_attrs.schachomeorganizationtype := owa_util.get_cgi_env('schachomeorganizationtype');
  plog.debug('schachomeorganizationtype:'||l_shibb_attrs.schachomeorganizationtype);
  l_shibb_attrs.schachomeorganization := owa_util.get_cgi_env('schachomeorganization');
  plog.debug('schachomeorganization:'||l_shibb_attrs.schachomeorganization);
  l_shibb_attrs.edupersonprincipalname := owa_util.get_cgi_env('edupersonprincipalname');
  plog.debug('edupersonprincipalname:'||l_shibb_attrs.edupersonprincipalname);
  l_shibb_attrs.edupersonaffiliation := owa_util.get_cgi_env('edupersonaffiliation');
  plog.debug('edupersonaffiliation:'||l_shibb_attrs.edupersonaffiliation);
  l_shibb_attrs.edupersontargetedid := owa_util.get_cgi_env('edupersontargetedid');
  plog.debug('edupersontargetedid:'||l_shibb_attrs.edupersontargetedid);
  l_shibb_attrs.elabainstitutionid := owa_util.get_cgi_env('elabainstitutionid');
  plog.debug('elabainstitutionid:'||l_shibb_attrs.elabainstitutionid);
  l_shibb_attrs.elabaprivileges := owa_util.get_cgi_env('elabaprivileges');
  plog.debug('elabaprivileges:'||l_shibb_attrs.elabaprivileges);
  l_shibb_attrs.elabadutydepartments := nvl(owa_util.get_cgi_env('elabadutydepartments'),
                                             owa_util.get_cgi_env('elabastudydepartments')); /*2014.03.25*/
  plog.debug('elabadutydepartments,elabastudydepartments:'||l_shibb_attrs.elabadutydepartments);

  return l_shibb_attrs;
end;

/*
  UNIT tests
*/
function test_get_tutor_attributes return t_attributes is

  l_result shibboleth.t_attributes;
begin
  l_result.uid := '211950810/767';
  l_result.commonname := 'Vardas Pavarde';
  l_result.surname := 'Pavarde';
  l_result.givenname := 'Vardas';
  l_result.displayname := 'Vardas Pavarde';
  l_result.email := 'vu.admin@asseco.lt';
  l_result.schachomeorganizationtype := 'urn:mace:terena.org:schac:homeOrganizationType:lt:university';
  l_result.schachomeorganization := '211950810';
  l_result.edupersonprincipalname := 'vu.admin@asseco.lt';
  l_result.edupersonaffiliation := 'employee';
  l_result.edupersontargetedid := '767';
  l_result.elabainstitutionid := '17';
  l_result.elabaprivileges := 'DOCUMENT__SUPERVISOR__EDIT_EPAS,DOCUMENT__REVIEWER__EDIT_EPAS,DOCUMENT__DEPARTMENT__EDIT_EPAS,DOCUMENT__INSTITUTION__EDIT_EPAS';
  l_result.elabadutydepartments := '119;117;115;123;121';
  return l_result;
end;

function test_get_stud_attributes return t_attributes is

  l_result shibboleth.t_attributes;
begin
  l_result.uid := '211950810/780';
  l_result.commonname := 'Vardas Pavarde';
  l_result.surname := 'Pavarde';
  l_result.givenname := 'Vardas';
  l_result.displayname := 'Vardas Pavarde';
  l_result.email := 'vu.stud@asseco.lt';
  l_result.schachomeorganizationtype := 'urn:mace:terena.org:schac:homeOrganizationType:lt:university';
  l_result.schachomeorganization := '211950810';
  l_result.edupersonprincipalname := 'vu.stud@asseco.lt';
  l_result.edupersonaffiliation := 'student';
  l_result.edupersontargetedid := '780';
  l_result.elabainstitutionid := '17';
  l_result.elabaprivileges := null;
  l_result.elabadutydepartments := '119';
  return l_result;
end;

end shibboleth;

/
