rem ---------------------------------------------------------------------------------
rem Rašto darbų mainavietės duomenų bazės objektų ištrinimo SQL komandinis 
rem failas
rem Autorius: Žilvinas Liaugaudas (UAB Affecto Lietuva)
rem Data: 2013-08-06
rem Modifikacijos (kada, kas, ką):
rem
rem ---------------------------------------------------------------------------------

set echo off
set verify off
set serveroutput on
set pagesize 0

rem ---------------------------------------------------------------------------------
rem Apibrėžiame pagrindinius sistemos parametrus
rem ---------------------------------------------------------------------------------

@@defines.sql

set feedback on
spool uninstall_.log

whenever sqlerror exit failure

prompt Stabdomi Oracle DB veikiantys sistemos foniniai darbai ...
declare
  cursor c_job(
    p_username in varchar2) is
  select /*+all_rows*/
    *
  from
    dba_jobs
  where
    log_user = p_username;    
begin
  for l_job in c_job(upper('&owner_username'))
  loop
    dbms_ijob.remove(l_job.job);
  end loop;
  commit;
end;
/

prompt Laukiama kol baigs veikti sistemos Oracle DB foniniai darbai ...
declare

  function is_job_running(
    p_username in varchar2) return boolean is
      
    cursor c_job(
      p_username in varchar2) is
    select /*+first_rows(1)*/ 
      '1' 
    from 
      v_$session s,
      dba_jobs_running j
    where
      s.username = p_username and
      s.sid = j.sid;
      
    l_dummy varchar2(1);  
  begin
    open c_job(upper(p_username));
    fetch c_job into l_dummy;
    close c_job;
    return l_dummy is not null;
  end;    
    
begin
  if is_job_running('&owner_username') then
    dbms_lock.sleep(15);
    if is_job_running('&owner_username') then
      raise_application_error(-20000, 'Nepavyko sulaukti sistemos foninių darbų veikimo pabaigos. Pabandykite vėliau.');
    end if;
  end if;
end;
/

rem ---------------------------------------------------------------------------------
rem Ištriname Oracle DB naudotoją - sistemos duomenų bazės objektų savininką
rem ---------------------------------------------------------------------------------

drop user &owner_username cascade;

rem ---------------------------------------------------------------------------------
rem Ištriname lentelių erdves
rem ---------------------------------------------------------------------------------

prompt Trinama lentelių erdvė "&STAB_TBS_NAME" ...
drop tablespace &STAB_TBS_NAME including contents and datafiles
/

prompt Trinama lentelių erdvė "&SIND_TBS_NAME" ...
drop tablespace &SIND_TBS_NAME including contents and datafiles
/

prompt Trinama lentelių erdvė "&MTAB_TBS_NAME" ...
drop tablespace &MTAB_TBS_NAME including contents and datafiles
/

prompt Trinama lentelių erdvė "&MIND_TBS_NAME" ...
drop tablespace &MIND_TBS_NAME including contents and datafiles
/

prompt Trinama lentelių erdvė "&LOBD_TBS_NAME" ...
drop tablespace &LOBD_TBS_NAME including contents and datafiles
/


begin
dbms_network_acl_admin.drop_acl(
    acl => 'owner_acl.xml');
  commit;
end;
/

spool off
quit

rem ---------------------------------------------------------------------------------
rem Failo pabaiga
rem ---------------------------------------------------------------------------------

